
//#define debug
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Globalization;
using System.Windows.Forms;
using System.IO.Ports;
using System.IO;
using System.Threading;


namespace XSOC_Gui
{
    public partial class guiForm : Form
    {
        // Constants
        const UInt16 BP_LOGIC_BASE_ADDR = 0x6004;
        const UInt16 SAVE_REG_ADDR = 0x0072;
        const string PROGRAM_NAME = "Logsys XSOC Software Debugger";
        
        private string LCCDIR; 
        private string CRT3; 
        private string LIBXR16; 


        // Forms (Settings Menu Items)
        private IOAddressesForm ioAddrForm;
        private UserIONamesForm userIONamesForm;
        
#if debug
        // Console for debugging
        private ConsoleForm cf = new ConsoleForm();
#endif

        // Serial Port handling
        private bool readWordSet = false;
        private bool readByteSet = false;
        private UInt16 receivedWord = 0;
        private Byte receivedByte = 0;
            // For thread synchronization
        static AutoResetEvent receivedValidSerialDataEvent = new AutoResetEvent(false);
        static AutoResetEvent escReceivedEvent = new AutoResetEvent(false);
        delegate void SetTextCallback(string str);
     
        // File opening
        private string fullFileName = "";
        private string fileName = "";
        private bool asmOpened = false;

        // File editing
        private bool alreadySaved = false;
        private bool sourceTextChanged = false;

        // Breakpoint handling
        private int breakPointLine = -2;
        private bool breakPointSet = false;
        private UInt16 breakPointAddress = 0;

        // Program running/halted GUI handling
        private int findIndex = 0;
        private int actualLine = -2;
        private bool steppedOnBreakPoint = false;
        private bool autoStepRunning = false;
        private bool programRunning = false;

        // IO Tab handling
        private int activeTabIndex = 0;
        private string homeDirectory = Directory.GetCurrentDirectory();
        private bool firstTimeRun = true;
        private int[,] userIOAddressOffsetValues = new int[4, 3];
        bool[,] sizeWordArray = new bool[4, 4];

        // MemoryGroupBox handling
        private UInt16 baseAddr = 0;
        private int startPosition = 0;
        private UInt16 memoryIndex = 0;

        // RegisterGroupBox
        private Register[] regs = new Register[15];
        private UInt16[] regValues = new UInt16[15];    //for reading memory values

        // Default IO base addresses
        private UInt16 itcAddr = 0x6000;
        private UInt16 switchAddr = 0x6020;
        private UInt16 ledAddr = 0x6030;
        private UInt16 segmentAddr = 0x6040;
        private UInt16 sramAddr = 0x6060;
        private UInt16 userIO1_Addr = 0x6100;
        private UInt16 userIO2_Addr = 0x6200;
        private UInt16 userIO3_Addr = 0x6300;
        private UInt16 userIO4_Addr = 0x6400;


        // get/set for Addresses -> sets Textbox.Text values
        private UInt16 ITCAddr
        {
            get { return itcAddr; }
            set
            {
                itcAddr = value;
                ITCAddrTextBox.Text = itcAddr.ToString("X04");
            }
        }
        private UInt16 SwitchAddr
        {
            get { return switchAddr; }
            set
            {
                switchAddr = value;
                SwitchAddressTextBox.Text = switchAddr.ToString("X04");
            }
        }
        private UInt16 LEDAddr
        {
            get { return ledAddr; }
            set
            {
                ledAddr = value;
                LEDAddrTextBox.Text = ledAddr.ToString("X04");
            }
        }
        private UInt16 SegmentAddr
        {
            get { return segmentAddr; }
            set
            {
                segmentAddr = value;
                SegmentAddrTextBox.Text = segmentAddr.ToString("X04");
            }
        }
        private UInt16 SRAMAddr
        {
            get { return sramAddr; }
            set
            {
                sramAddr = value;
                SRAMAddrTextBox.Text = sramAddr.ToString("X04");
            }
        }
        private UInt16 UserIO1_Addr
        {
            get { return userIO1_Addr; }
            set
            {
                userIO1_Addr = value;
                UserIO1_AddrTextBox.Text = userIO1_Addr.ToString("X04");
            }
        }
        private UInt16 UserIO2_Addr
        {
            get { return userIO2_Addr; }
            set
            {
                userIO2_Addr = value;
                UserIO2_AddrTextBox.Text = userIO2_Addr.ToString("X04");
            }
        }
        private UInt16 UserIO3_Addr
        {
            get { return userIO3_Addr; }
            set
            {
                userIO3_Addr = value;
                UserIO3_AddrTextBox.Text = userIO3_Addr.ToString("X04");
            }
        }
        private UInt16 UserIO4_Addr
        {
            get { return userIO4_Addr; }
            set
            {
                userIO4_Addr = value;
                UserIO4_AddrTextBox.Text = userIO4_Addr.ToString("X04");
            }
        }


        // Constructor
        public guiForm()
        {
            InitializeComponent();
#if debug
            cf.Show();
#endif
            this.Text = PROGRAM_NAME;
            for (int i = 0; i < regs.Length; i++)
                regs[i] = new Register();

            
            LCCDIR = Environment.GetEnvironmentVariable("LCCDIR");
            CRT3 = LCCDIR + "\\CRT3.s ";
            LIBXR16 = LCCDIR + "\\libxr16.s ";

            // Assign guiForm's TextBoxes to Register's TextBoxes
            regs[0].RegTextBox = rTextBox1;
            regs[1].RegTextBox = rTextBox2;
            regs[2].RegTextBox = rTextBox3;
            regs[3].RegTextBox = rTextBox4;
            regs[4].RegTextBox = rTextBox5;
            regs[5].RegTextBox = rTextBox6;
            regs[6].RegTextBox = rTextBox7;
            regs[7].RegTextBox = rTextBox8;
            regs[8].RegTextBox = rTextBox9;
            regs[9].RegTextBox = rTextBox10;
            regs[10].RegTextBox = rTextBox11;
            regs[11].RegTextBox = rTextBox12;
            regs[12].RegTextBox = rTextBox13;
            regs[13].RegTextBox = rTextBox14;
            regs[14].RegTextBox = rTextBox15;

#if  debug

            cf.ConsoleWriteLine("working directory: " + homeDirectory);
#endif
            compilerProcess.StartInfo.FileName = LCCDIR + "\\lcc-xr16.exe";
            assemblerProcess.StartInfo.FileName = LCCDIR + "\\xr16.exe";
            allowFunctionsInToolStripForNotSaved();
            setIOGroupBoxValues();

        }

        //--------------------IO GROUPBOX------------------------------
        private void initXsocDatFile()
        {
            bool readOnly = true;
            bool sizeWord = true;

            FileStream datFile = new FileStream(homeDirectory + "\\xsoc.dat", FileMode.CreateNew);
            BinaryWriter w = new BinaryWriter(datFile);
            w.Write(SwitchAddr);
            w.Write(LEDAddr);
            w.Write(SegmentAddr);
            w.Write(SRAMAddr);
            w.Write(ITCAddr);
            w.Write(UserIO1_Addr);
            w.Write(UserIO2_Addr);
            w.Write(UserIO3_Addr);
            w.Write(UserIO4_Addr);

            for (int i = 1; i < 5; i++)
            {
                w.Write("UserIO " + i.ToString());
                for (int j = 1; j < 5; j++)
                {
                    w.Write("Reg" + j.ToString());
                    w.Write(readOnly);
                    w.Write(sizeWord);
                }
            }

            w.Close();
            datFile.Close();
        }


        private void setIOGroupBoxValues()
        {
#if  debug
            cf.ConsoleWriteLine("setIOGroupBoxValues");
#endif
            FileStream datFile;
            bool readOnly = true;
            bool sizeWord = true;

            int index;

            TextBox[,] regTextboxes = new TextBox[4, 4];
            Label[,] regLabels = new Label[4, 7];

            regLabels[0, 0] = UserIO1_Reg1Label;
            regLabels[0, 1] = UserIO1_Reg2Label;
            regLabels[0, 2] = UserIO1_Reg3Label;
            regLabels[0, 3] = UserIO1_Reg4Label;
            regLabels[0, 4] = userIO1_reg2AddrLabel;
            regLabels[0, 5] = userIO1_reg3AddrLabel;
            regLabels[0, 6] = userIO1_reg4AddrLabel;

            regLabels[1, 0] = UserIO2_Reg1Label;
            regLabels[1, 1] = UserIO2_Reg2Label;
            regLabels[1, 2] = UserIO2_Reg3Label;
            regLabels[1, 3] = UserIO2_Reg4Label;
            regLabels[1, 4] = userIO2_reg2AddrLabel;
            regLabels[1, 5] = userIO2_reg3AddrLabel;
            regLabels[1, 6] = userIO2_reg4AddrLabel;

            regLabels[2, 0] = UserIO3_Reg1Label;
            regLabels[2, 1] = UserIO3_Reg2Label;
            regLabels[2, 2] = UserIO3_Reg3Label;
            regLabels[2, 3] = UserIO3_Reg4Label;
            regLabels[2, 4] = userIO3_reg2AddrLabel;
            regLabels[2, 5] = userIO3_reg3AddrLabel;
            regLabels[2, 6] = userIO3_reg4AddrLabel;

            regLabels[3, 0] = UserIO4_Reg1Label;
            regLabels[3, 1] = UserIO4_Reg2Label;
            regLabels[3, 2] = UserIO4_Reg3Label;
            regLabels[3, 3] = UserIO4_Reg4Label;
            regLabels[3, 4] = userIO4_reg2AddrLabel;
            regLabels[3, 5] = userIO4_reg3AddrLabel;
            regLabels[3, 6] = userIO4_reg4AddrLabel;

            regTextboxes[0, 0] = UserIO1_Reg1TextBox;
            regTextboxes[0, 1] = UserIO1_Reg2TextBox;
            regTextboxes[0, 2] = UserIO1_Reg3TextBox;
            regTextboxes[0, 3] = UserIO1_Reg4TextBox;

            regTextboxes[1, 0] = UserIO2_Reg1TextBox;
            regTextboxes[1, 1] = UserIO2_Reg2TextBox;
            regTextboxes[1, 2] = UserIO2_Reg3TextBox;
            regTextboxes[1, 3] = UserIO2_Reg4TextBox;

            regTextboxes[2, 0] = UserIO3_Reg1TextBox;
            regTextboxes[2, 1] = UserIO3_Reg2TextBox;
            regTextboxes[2, 2] = UserIO3_Reg3TextBox;
            regTextboxes[2, 3] = UserIO3_Reg4TextBox;

            regTextboxes[3, 0] = UserIO4_Reg1TextBox;
            regTextboxes[3, 1] = UserIO4_Reg2TextBox;
            regTextboxes[3, 2] = UserIO4_Reg3TextBox;
            regTextboxes[3, 3] = UserIO4_Reg4TextBox;

            if (!File.Exists(homeDirectory + "\\xsoc.dat"))
            {
                initXsocDatFile();
                MessageBox.Show("Not able to open xsoc.dat\r\n\r\nLoading default values",
                                "Data File Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }

            datFile = new FileStream(homeDirectory + "\\xsoc.dat", FileMode.Open, FileAccess.Read);
            BinaryReader br = new BinaryReader(datFile);

            // Addresses
            SwitchAddr = br.ReadUInt16();
            LEDAddr = br.ReadUInt16();
            SegmentAddr = br.ReadUInt16();
            SRAMAddr = br.ReadUInt16();
            ITCAddr = br.ReadUInt16();
            UserIO1_Addr = br.ReadUInt16();
            UserIO2_Addr = br.ReadUInt16();
            UserIO3_Addr = br.ReadUInt16();
            UserIO4_Addr = br.ReadUInt16();

            // User IO Names: IO Name, REG1(Name,ReadOnly,sizeWord),REG2,REG3,REG4

            for (int i = 0; i < 4; i++)
            {
                index = 0;
                if (i == 0)
                    UserIOTabPage1.Text = br.ReadString();
                else if (i == 1)
                    UserIOTabPage2.Text = br.ReadString();
                else if (i == 2)
                    UserIOTabPage3.Text = br.ReadString();
                else
                    UserIOTabPage4.Text = br.ReadString();

                for (int j = 0; j < 4; j++)
                {
                    regLabels[i, j].Text = br.ReadString();
                    readOnly = br.ReadBoolean();
                    sizeWord = br.ReadBoolean();
                    sizeWordArray[i, j] = sizeWord;
                    regTextboxes[i, j].MaxLength = sizeWord ? 4 : 2;
                    regTextboxes[i, j].Width = sizeWord ? 35 : 22;
                    regTextboxes[i, j].ReadOnly = readOnly;

                    if (j == 0)
                        index = sizeWordArray[i, j] ? 2 : 1;
                    else
                    {
                        if (sizeWordArray[i, j] && (index % 2 == 1))
                            index++;
                        regLabels[i, j + 3].Text = "+" + index.ToString();
                        userIOAddressOffsetValues[i, j - 1] = index;
                        index += sizeWordArray[i, j] ? 2 : 1;
                    }
                }
            }

            br.Close();
            datFile.Close();

            if (!firstTimeRun)
                updateIOTabValues(activeTabIndex);

            firstTimeRun = false;
        }

        private void ioDevicesTabControl_Selected(object sender, TabControlEventArgs e)
        {
            updateIOTabValues(e.TabPageIndex);
            activeTabIndex = e.TabPageIndex;
        }
        private void updateIOTabValues(int tabIndex)
        {

            UInt16 readAddress;
            try
            {

                switch (tabIndex)
                {
                    case 0: // XIO - Switch, Led
                        {
                            SwitchDataTextBox.Text = debmonReadByte(SwitchAddr).ToString("X02");
                            LEDDataTextBox.Text = debmonReadByte(LEDAddr).ToString("X02");
                            break;
                        }
                    case 1: //SEGMENT
                        {
                            SegmentAnode0TextBox.Text = debmonReadByte(SegmentAddr).ToString("X02");
                            SegmentAnode1TextBox.Text = debmonReadByte((UInt16)(SegmentAddr + 1)).ToString("X02");
                            SegmentAnode2TextBox.Text = debmonReadByte((UInt16)(SegmentAddr + 2)).ToString("X02");
                            SegmentAnode3TextBox.Text = debmonReadByte((UInt16)(SegmentAddr + 3)).ToString("X02");
                            SegmentHexDataTextBox.Text = debmonReadWord((UInt16)(SegmentAddr + 4)).ToString("X04");
                            if ((debmonReadByte((UInt16)(segmentAddr + 6)) & (Byte)0x1) == 1)
                                SegmentAnodeCheckBox.Checked = true;
                            else
                                SegmentAnodeCheckBox.Checked = false;


                            break;
                        }
                    case 2: // SRAM
                        {
                            SRAMPageTextBox.Text = debmonReadByte(SRAMAddr).ToString("X02");
                            break;
                        }
                    case 3: // ITC
                        {
                            ITCJumpVectorTextBox.Text = debmonReadWord(ITCAddr).ToString("X04");
                            ITCEnableTextBox.Text = debmonReadByte((UInt16)(ITCAddr + 2)).ToString("X02");
                            ITCStatusTextBox.Text = debmonReadByte((UInt16)(ITCAddr + 3)).ToString("X02");
                            break;
                        }
                    case 4: // USER IO 1
                        {

                            if (sizeWordArray[0, 0])
                                UserIO1_Reg1TextBox.Text = debmonReadWord(UserIO1_Addr).ToString("X04");
                            else
                                UserIO1_Reg1TextBox.Text = debmonReadByte(UserIO1_Addr).ToString("X02");

                            readAddress = (UInt16)(UserIO1_Addr + userIOAddressOffsetValues[0, 0]);
                            if (sizeWordArray[0, 1])
                                UserIO1_Reg2TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO1_Reg2TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO1_Addr + userIOAddressOffsetValues[0, 1]);
                            if (sizeWordArray[0, 2])
                                UserIO1_Reg3TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO1_Reg3TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO1_Addr + userIOAddressOffsetValues[0, 2]);
                            if (sizeWordArray[0, 3])
                                UserIO1_Reg4TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO1_Reg4TextBox.Text = debmonReadByte(readAddress).ToString("X02");


                            break;
                        }
                    case 5: // USER IO 2
                        {

                            if (sizeWordArray[1, 0])
                                UserIO2_Reg1TextBox.Text = debmonReadWord(UserIO2_Addr).ToString("X04");
                            else
                                UserIO2_Reg1TextBox.Text = debmonReadByte(UserIO2_Addr).ToString("X02");

                            readAddress = (UInt16)(UserIO2_Addr + userIOAddressOffsetValues[1, 0]);
                            if (sizeWordArray[1, 1])
                                UserIO2_Reg2TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO2_Reg2TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO2_Addr + userIOAddressOffsetValues[1, 1]);
                            if (sizeWordArray[1, 2])
                                UserIO2_Reg3TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO2_Reg3TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO2_Addr + userIOAddressOffsetValues[1, 2]);
                            if (sizeWordArray[1, 3])
                                UserIO2_Reg4TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO2_Reg4TextBox.Text = debmonReadByte(readAddress).ToString("X02");


                            break;
                        }
                    case 6: // USER IO 3
                        {
                            if (sizeWordArray[2, 0])
                                UserIO3_Reg1TextBox.Text = debmonReadWord(UserIO3_Addr).ToString("X04");
                            else
                                UserIO3_Reg1TextBox.Text = debmonReadByte(UserIO3_Addr).ToString("X02");

                            readAddress = (UInt16)(UserIO3_Addr + userIOAddressOffsetValues[2, 0]);
                            if (sizeWordArray[2, 1])
                                UserIO3_Reg2TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO3_Reg2TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO3_Addr + userIOAddressOffsetValues[2, 1]);
                            if (sizeWordArray[2, 2])
                                UserIO3_Reg3TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO3_Reg3TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO3_Addr + userIOAddressOffsetValues[2, 2]);
                            if (sizeWordArray[2, 3])
                                UserIO3_Reg4TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO3_Reg4TextBox.Text = debmonReadByte(readAddress).ToString("X02");
                            break;
                        }
                    case 7: // USER IO 4
                        {
                            if (sizeWordArray[3, 0])
                                UserIO4_Reg1TextBox.Text = debmonReadWord(UserIO4_Addr).ToString("X04");
                            else
                                UserIO4_Reg1TextBox.Text = debmonReadByte(UserIO4_Addr).ToString("X02");

                            readAddress = (UInt16)(UserIO4_Addr + userIOAddressOffsetValues[3, 0]);
                            if (sizeWordArray[3, 1])
                                UserIO4_Reg2TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO4_Reg2TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO4_Addr + userIOAddressOffsetValues[3, 1]);
                            if (sizeWordArray[3, 2])
                                UserIO4_Reg3TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO4_Reg3TextBox.Text = debmonReadByte(readAddress).ToString("X02");

                            readAddress = (UInt16)(UserIO4_Addr + userIOAddressOffsetValues[3, 2]);
                            if (sizeWordArray[3, 3])
                                UserIO4_Reg4TextBox.Text = debmonReadWord(readAddress).ToString("X04");
                            else
                                UserIO4_Reg4TextBox.Text = debmonReadByte(readAddress).ToString("X02");
                            break;

                        }

                }
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Debug serial port is not open\r\nChoose a port!",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            catch
            {
                MessageBox.Show("Serial Port timed out\r\nCheck Port settings",
                                "Serial Port Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }

        }
       
        private void SegmentAnodeCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UInt16 segmentStatusAddr = (UInt16)(segmentAddr + 6);
            if (SegmentAnodeCheckBox.Checked)
            {
                SegmentHexDataTextBox.ReadOnly = true;
                SegmentAnode0TextBox.ReadOnly = false;
                SegmentAnode1TextBox.ReadOnly = false;
                SegmentAnode2TextBox.ReadOnly = false;
                SegmentAnode3TextBox.ReadOnly = false;

                debmonWriteByte(segmentStatusAddr, 1);

            }
            else
            {
                SegmentHexDataTextBox.ReadOnly = false;
                SegmentAnode0TextBox.ReadOnly = true;
                SegmentAnode1TextBox.ReadOnly = true;
                SegmentAnode2TextBox.ReadOnly = true;
                SegmentAnode3TextBox.ReadOnly = true;

                debmonWriteByte(segmentStatusAddr, 0);

            }

        }
        private void SegmentAnodeTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            byte setByte, getByte;
            UInt16 sendAddr;
            TextBox senderTextBox = (TextBox)sender;
            if (senderTextBox.Equals(SegmentAnode0TextBox))
                sendAddr = (UInt16)(segmentAddr + 0);
            else if (senderTextBox.Equals(SegmentAnode1TextBox))
                sendAddr = (UInt16)(segmentAddr + 1);
            else if (senderTextBox.Equals(SegmentAnode2TextBox))
                sendAddr = (UInt16)(segmentAddr + 2);
            else
                sendAddr = (UInt16)(segmentAddr + 3);

            if (e.KeyChar == (char)Keys.Enter)
            {
                if (Byte.TryParse(senderTextBox.Text, NumberStyles.HexNumber, null, out setByte))
                {
                    debmonWriteByte(sendAddr, setByte);
                }

                getByte = debmonReadByte(sendAddr);
                senderTextBox.Text = getByte.ToString("X02");
            }
        }
        private void SegmentHexDataTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            UInt16 setValue, getValue;
            if (e.KeyChar == (char)Keys.Enter)
            {

                if (UInt16.TryParse(SegmentHexDataTextBox.Text, NumberStyles.HexNumber, null, out setValue))
                {
                    debmonWriteWord((UInt16)(segmentAddr + 4), setValue);
                }

                getValue = debmonReadWord((UInt16)(segmentAddr + 4));
                SegmentHexDataTextBox.Text = getValue.ToString("X04");
            }
        }
        private void LEDDataTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            Byte setValue, getValue;
            if (e.KeyChar == (char)Keys.Enter)
            {

                if (Byte.TryParse(LEDDataTextBox.Text, NumberStyles.HexNumber, null, out setValue))
                {
                    debmonWriteByte(LEDAddr, setValue);
                }

                getValue = debmonReadByte(LEDAddr);
                LEDDataTextBox.Text = getValue.ToString("X02");
            }
        }
        private void SRAMPageTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            Byte setValue, getValue;
            if (e.KeyChar == (char)Keys.Enter)
            {

                if (Byte.TryParse(SRAMPageTextBox.Text, NumberStyles.HexNumber, null, out setValue))
                {
#if  debug
                    cf.ConsoleWriteLine("SRAMPageTextBox_KeyPress - setValue: " + setValue.ToString("X02"));
#endif
                    debmonWriteByte(SRAMAddr, setValue);
                }

                getValue = debmonReadByte(SRAMAddr);
                SRAMPageTextBox.Text = getValue.ToString("X02");
            }
        }
        private void ITCEnableTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            Byte setValue, getValue;
            if (e.KeyChar == (char)Keys.Enter)
            {

                if (Byte.TryParse(ITCEnableTextBox.Text, NumberStyles.HexNumber, null, out setValue))
                {
                    setValue |= 1;  // also bit mindig enable -> jobb lenne HW-bl
                    debmonWriteByte(ITCAddr, setValue);
                }

                getValue = debmonReadByte(ITCAddr);
                ITCEnableTextBox.Text = getValue.ToString("X02");
            }
        }
        private void UserIO_RegTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            UInt16 setValueUInt16, getValueUInt16;
            Byte setValueByte, getValueByte;
            UInt16 regAddr;
            int ioNum;
            bool sizeWord;
            TextBox tb = (TextBox)sender;


            if (tb.Name.Contains("UserIO1"))
            {
                regAddr = UserIO1_Addr;
                ioNum = 0;
            }
            else if (tb.Name.Contains("UserIO2"))
            {
                regAddr = UserIO2_Addr;
                ioNum = 1;
            }
            else if (tb.Name.Contains("UserIO3"))
            {
                regAddr = UserIO3_Addr;
                ioNum = 2;
            }
            else
            {
                regAddr = UserIO4_Addr;
                ioNum = 3;
            }

            if (tb.Name.Contains("Reg1"))
                sizeWord = sizeWordArray[ioNum, 0];
            else if (tb.Name.Contains("Reg2"))
            {
                regAddr += (UInt16)userIOAddressOffsetValues[ioNum, 0];
                sizeWord = sizeWordArray[ioNum, 1];
            }
            else if (tb.Name.Contains("Reg3"))
            {
                regAddr += (UInt16)userIOAddressOffsetValues[ioNum, 1];
                sizeWord = sizeWordArray[ioNum, 2];
            }
            else
            {
                regAddr += (UInt16)userIOAddressOffsetValues[ioNum, 2];
                sizeWord = sizeWordArray[ioNum, 3];
            }

            if (e.KeyChar == (char)Keys.Enter)
            {
                if (sizeWord)
                {
                    if (UInt16.TryParse(tb.Text, NumberStyles.HexNumber, null, out setValueUInt16))
                    {
                        debmonWriteWord(regAddr, setValueUInt16);
                    }
                    getValueUInt16 = debmonReadWord(regAddr);
                    tb.Text = getValueUInt16.ToString("X04");

                }
                else
                {
                    if (Byte.TryParse(tb.Text, NumberStyles.HexNumber, null, out setValueByte))
                    {
                        debmonWriteByte(regAddr, setValueByte);
                    }

                    getValueByte = debmonReadByte(regAddr);
                    tb.Text = getValueByte.ToString("X02");
                }




            }



        }




        // BreakPoint placement

        private void lstRichTextBox_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            int index;
            int first_ch;
            int last_ch;
            int lineNum;

            index = lstRichTextBox.GetCharIndexFromPosition(e.Location);
            lineNum = lstRichTextBox.GetLineFromCharIndex(index);
            lstRichTextBox.Enabled = false;
            
            if (breakPointSet)
            {
                first_ch = lstRichTextBox.GetFirstCharIndexFromLine(breakPointLine);
                last_ch = lstRichTextBox.Text.IndexOf("\n", first_ch);

                lstRichTextBox.Select(first_ch, last_ch - first_ch);
                if (breakPointLine != actualLine)
                {
                    lstRichTextBox.SelectionBackColor = lstRichTextBox.BackColor;
                    lstRichTextBox.SelectionColor = lstRichTextBox.ForeColor;
                }
                else
                {
                    lstRichTextBox.SelectionBackColor = Color.Yellow;
                    lstRichTextBox.SelectionColor = lstRichTextBox.ForeColor;
                    steppedOnBreakPoint = false;
                }


            }

            first_ch = lstRichTextBox.GetFirstCharIndexFromLine(lineNum);
            last_ch = lstRichTextBox.Text.IndexOf("\n", first_ch);
            lstRichTextBox.Select(first_ch, last_ch - first_ch);
            if (lstRichTextBox.SelectedText.Length > 5 &&
                UInt16.TryParse(lstRichTextBox.SelectedText.Substring(0, 4), NumberStyles.HexNumber, null, out breakPointAddress))
            {
                if (!(lineNum == breakPointLine && breakPointSet))
                {
                    breakPointSet = true;
                    breakPointLine = lineNum;
                    lstRichTextBox.SelectionColor = Color.White;
                    if (breakPointLine != actualLine)
                    {
                        lstRichTextBox.SelectionBackColor = Color.DarkRed;
                        
                    }
                    else
                    {
                        lstRichTextBox.SelectionBackColor = Color.DarkOrange;
                        steppedOnBreakPoint = true;
                    }

                    
                }else
                    breakPointSet = false;

            }
            else
            {
                breakPointSet = false;
                lstRichTextBox.Enabled = true;
                MessageBox.Show("Unable to set breakpoint",
                                "Set Breakpoint Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation);
            }


            lstRichTextBox.DeselectAll();
            lstRichTextBox.Enabled = true;
            lstRichTextBox.Focus();

        }



        //--------------------MEMORY GROUPBOX------------------------------
        private void readMemoryButton_Click(object sender, EventArgs e)
        {
            UInt16 readVal;
            bool oddMemory = false;
            int i, j,inc;
            string str;
            string outputText = String.Empty;

            // Address textbox

            if (UInt16.TryParse(addrTextBox.Text, NumberStyles.HexNumber, null, out readVal))
            {
                if (readVal % 2 == 1)
                {
                    --readVal;
                    //WriteLineToConsole("Cannot set odd memory address!");
#if  debug
                    cf.ConsoleWriteLine("Cannot set odd memory address!");
#endif
                    oddMemory = true;
                    //processToolStripStatusLabel.Text = "Cannot set odd memory address!";
                }

                baseAddr = readVal;

                //WriteLineToConsole("Read memory values from " + baseAddr.ToString("X04") + " + 48 byte");

                if (oddMemory)
                    processToolStripStatusLabel.Text += "  (Cannot set odd memory address)";

                // Word/Byte RadioButtons

                if (wordRadioButton.Checked)
                {
                    inc = 5;
                    editorTextBox.MaxLength = 4;
                    editorTextBox.Size = new Size(40, 20);
                    editorTextBox.Visible = false;
                }
                else
                {
                    inc = 3;
                    editorTextBox.Visible = false;
                    editorTextBox.MaxLength = 2;
                    editorTextBox.Size = new Size(25, 20);
                }
                for (j = 0; j < memoryTextBox.Lines.Length; j++)
                {
                    str = memoryTextBox.Lines[j];
                    str = str.Replace(" ", "");

                    for (i = 5; i < str.Length; i += inc)
                    {
                        str = str.Insert(i, " ");

                    }
                    outputText += str;
                    if (j < memoryTextBox.Lines.Length - 1)
                         outputText += "\r\n";
                    
                }
                memoryTextBox.Text = outputText;
#if  debug
                cf.ConsoleWriteLine(outputText.Length.ToString());
#endif
                memoryTextBox.Refresh();
                readMemoryValues();
            }
            else
            {
                MessageBox.Show("Wrong address!",
                                "MemoryBox Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error
                                );
            }



        }

        private void RadioButton_CheckedChanged(object sender, EventArgs e)
        {
            //int i,j;
            //int first;
            //int last;
            //string str;


            //if (wordRadioButton.Checked)
            //{


            //    for (j = 0; j < 12; j++)
            //    {
            //        first = memoryTextBox.GetFirstCharIndexFromLine(j);
            //        if (j == 11)
            //            last = memoryTextBox.Text.Length - 1;
            //        else
            //            last = memoryTextBox.Text.IndexOf(Environment.NewLine, first);
            //        str = memoryTextBox.Text.Substring(first, last - first);
            //        str = str.Replace(" ", "");

            //        for (i = 5; i < str.Length; i += 5)
            //        {
            //            str = str.Insert(i, " ");

            //        }
            //        memoryTextBox.Text = memoryTextBox.Text.Remove(first, last - first);
            //        memoryTextBox.Text = memoryTextBox.Text.Insert(first, str);
            //    }


            //    editorTextBox.MaxLength = 4;
            //    editorTextBox.Size = new Size(40, 20);
            //    editorTextBox.Visible = false;



            //}
            //else
            //{

            //    for (j = 0; j < 12; j++)
            //    {
            //        first=memoryTextBox.GetFirstCharIndexFromLine(j);
            //        if (j == 11)
            //            last = memoryTextBox.Text.Length - 1;
            //        else
            //            last = memoryTextBox.Text.IndexOf(Environment.NewLine, first);
            //        str = memoryTextBox.Text.Substring(first,last-first);
            //        str = str.Replace(" ","");
            //        for (i = 5; i < str.Length; i += 3)
            //        {
            //            str = str.Insert(i, " ");

            //        }
            //        memoryTextBox.Text = memoryTextBox.Text.Remove(first, last - first);
            //        memoryTextBox.Text = memoryTextBox.Text.Insert(first, str);
            //    }
            //    editorTextBox.Visible = false;
            //    editorTextBox.MaxLength = 2;
            //    editorTextBox.Size = new Size(25, 20);


            //}



        }
        private void addrTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                readMemoryButton_Click(sender, EventArgs.Empty);
            }
        }
        private void editorTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            UInt16 setUInt16, getUInt16;
            Byte setByte, getByte;
            UInt16 sendAddr = (UInt16)(baseAddr + memoryIndex);

            if (e.KeyChar == (char)Keys.Enter)
            {

                try
                {

                    if (wordRadioButton.Checked)
                    {
                        if (UInt16.TryParse(editorTextBox.Text, NumberStyles.HexNumber, null, out setUInt16))
                        {
                            debmonWriteWord(sendAddr, setUInt16);
                            getUInt16 = debmonReadWord(sendAddr);
                            memoryTextBox.Text = memoryTextBox.Text.Remove(startPosition, 4);
                            memoryTextBox.Text = memoryTextBox.Text.Insert(startPosition, getUInt16.ToString("X04"));
                        }
                    }
                    else
                    {
                        if (Byte.TryParse(editorTextBox.Text, NumberStyles.HexNumber, null, out setByte))
                        {
                            debmonWriteByte(sendAddr, setByte);
                            getByte = debmonReadByte(sendAddr);
                            memoryTextBox.Text = memoryTextBox.Text.Remove(startPosition, 2);
                            memoryTextBox.Text = memoryTextBox.Text.Insert(startPosition, getByte.ToString("X02"));
                        }
                    }
                }
                catch (InvalidOperationException)
                {
                    MessageBox.Show("Debug serial port is not open\r\nChoose a port!  memtextbox",
                        "Serial Port Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
                catch
                {
                    MessageBox.Show("Serial Port timed out\r\nCheck Port settings",
                                    "Serial Port Error",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error);
                }
                finally
                {
                    editorTextBox.Visible = false;
                    memoryTextBox.Focus();
                    memoryTextBox.DeselectAll();
                }
            }
        }
        private void memoryTextBox_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            int idx;
            int sp_idx = -1;
            int line;
            int first;
            editorTextBox.Visible = false;
            idx = memoryTextBox.GetCharIndexFromPosition(e.Location);
            line = memoryTextBox.GetLineFromCharIndex(idx);
            first = memoryTextBox.GetFirstCharIndexFromLine(line);
            if (idx - first > 5)
            {
                if (wordRadioButton.Checked)
                    sp_idx = memoryTextBox.Text.IndexOf(" ", idx - 5, 5);
                else
                    sp_idx = memoryTextBox.Text.IndexOf(" ", idx - 3, 3);
            }

            if (sp_idx > 0)
            {
                startPosition = sp_idx + 1;
                if (wordRadioButton.Checked)
                {
                    editorTextBox.Text = memoryTextBox.Text.Substring(startPosition, 4);
                    memoryIndex = (UInt16)(((sp_idx - memoryTextBox.GetFirstCharIndexFromLine(line) - 5) / 5 + 4 * line) * 2);
                }
                else
                {
                    editorTextBox.Text = memoryTextBox.Text.Substring(startPosition, 2);
                    memoryIndex = (UInt16)((sp_idx - memoryTextBox.GetFirstCharIndexFromLine(line) - 5) / 3 + 8 * line);
                }
                editorTextBox.Location = memoryTextBox.GetPositionFromCharIndex(sp_idx + 1);
                editorTextBox.Visible = true;
                editorTextBox.Focus();
            }
        }
        private void editorTextBox_Leave(object sender, EventArgs e)
        {
            editorTextBox.Visible = false;
        }

        //--------------------REGISTER GROUPBOX----------------------------

        private void registersTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            int i;


            for (i = 0; i < regs.Length; i++)
            {
                if (regs[i].RegTextBox.Equals((TextBox)sender))
                    break;
            }

            try
            {
                if (e.KeyChar == (char)Keys.Enter)
                {
                    if (regs[i].TryParseRegTextBox())
                        debmonWriteWord((UInt16)(SAVE_REG_ADDR + i * 2), regs[i].RegValue);
                    else
                        processToolStripStatusLabel.Text = "Unable to set register";
                }
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Debug serial port is not open\r\nChoose a port! regtextbox",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }

        }
        private void registersContextMenuStrip_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            int i;



            for (i = 0; i < regs.Length; i++)
            {
                if (regs[i].RegTextBox == registersContextMenuStrip.SourceControl)
                    break;
            }

            regs[i].RegRadix = e.ClickedItem.ToString();
            //WriteLineToConsole("Regs[" + (i + 1).ToString() + "] :" + regs[i].ToString());
        }

        //----------------------TOOLSTRIP MENU-----------------------------

        private void newToolStripButton_Click(object sender, EventArgs e)
        {
            if (askForSaving() == 1)
                return;


            sourceRichTextBox.Clear();
            sourceRichTextBox.ReadOnly = false;
            saveToolStripButton.Enabled = true;
            lstRichTextBox.Clear();
            lstRichTextBox.Text = "\r\n\r\n                    LST File";
            this.Text = PROGRAM_NAME + " - Untitled";
            sourceTextChanged = false;
            alreadySaved = false;
            
        }

        private void saveToolStripButton_Click(object sender, EventArgs e)
        {
            if (alreadySaved)
            {
                if (sourceTextChanged)
                {
                    sourceRichTextBox.SaveFile(fullFileName, RichTextBoxStreamType.PlainText);
                    sourceTextChanged = false;
                    setGuiText(fullFileName);
                    lstRichTextBox.Clear();
                    lstRichTextBox.Text = "\r\n\r\n                    LST File";
                    allowFunctionsInToolStripForSaved();
                }
            }
            else
            {
                saveAsToolStripMenuItem_Click(sender, e);
            }
            
        }
        private void OpenLstFile(string file)
        {
            int start_pos;
            int end_pos;
            int first_index;
            string currentLine;
            bool errorFound = false;
            //System.Drawing.FontStyle newFontStyle = FontStyle.Bold;

            try
            {
                breakPointSet = false;


                lstRichTextBox.LoadFile(file, RichTextBoxStreamType.PlainText);

                lstRichTextBox.Enabled = false;
                for (int i = 0; i < lstRichTextBox.Lines.Length; i++)
			    {
                    currentLine = lstRichTextBox.Lines[i];
                    first_index = lstRichTextBox.GetFirstCharIndexFromLine(i);
                    start_pos = currentLine.IndexOf("_");
                    if (start_pos > 0 && currentLine.Replace(" ","").StartsWith("_"))
                    {
                        end_pos = currentLine.IndexOf(":", start_pos);
                        if (end_pos > 0)
                        {
                            
                            start_pos += first_index;
                            end_pos += first_index;
                            lstRichTextBox.Select(start_pos, end_pos - start_pos + 1);
                            lstRichTextBox.SelectionColor = Color.Red;
                            lstRichTextBox.SelectionFont = new Font(
                               lstRichTextBox.SelectionFont.FontFamily,
                               lstRichTextBox.SelectionFont.Size,
                               FontStyle.Bold
                            );
                        }
                    }

                    start_pos = currentLine.IndexOf(";");
                    if (start_pos > 0)
                    {
                        end_pos = currentLine.Length;
                        start_pos += first_index;
                        end_pos += first_index;
                        lstRichTextBox.Select(start_pos, end_pos - start_pos + 1);
                        lstRichTextBox.SelectionColor = Color.Green;
                        lstRichTextBox.SelectionFont = new Font(
                           lstRichTextBox.SelectionFont.FontFamily,
                           lstRichTextBox.SelectionFont.Size,
                           FontStyle.Regular
                        );
                    }

                    start_pos = currentLine.IndexOf("#");
                    if (start_pos > 0)
                    {
                        end_pos = currentLine.Length;
                        start_pos += first_index;
                        end_pos += first_index;
                        lstRichTextBox.Select(start_pos, end_pos - start_pos + 1);
                        lstRichTextBox.SelectionColor = Color.Blue;
                        lstRichTextBox.SelectionFont = new Font(
                           lstRichTextBox.SelectionFont.FontFamily,
                           lstRichTextBox.SelectionFont.Size,
                           FontStyle.Bold
                        );
                    }

                    if (currentLine.Contains("# error"))
                        errorFound=true;
                }
                               
                

                lstRichTextBox.DeselectAll();
                lstRichTextBox.Enabled = true;
                projectSourcesTabControl.SelectedIndex = 1;
                if (errorFound)
                    allowFunctionsInToolStripForSaved();
                else
                    allowFunctionsInToolStripForCompiled();
                    
            }
            catch
            {
                lstRichTextBox.Clear();
                lstRichTextBox.Text = "\r\n\r\n                    LST File";
                projectSourcesTabControl.SelectedIndex = 0;
            }

        }

        private void removeLocInfoFromAssembly()
        {
            string readLine;
            StreamReader sr = new StreamReader(fileName + ".s");
            StreamWriter sw = new StreamWriter(fileName + ".tmp");
            
            while ((readLine = sr.ReadLine()) != null)
            {
                if (!readLine.StartsWith("loc") && !readLine.StartsWith("file"))
                    sw.WriteLine(readLine);
            }
            
            sr.Close();
            sw.Close();
            File.Copy(fileName + ".tmp", fileName + ".s", true);
            File.Delete(fileName + ".tmp");
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string extension;
            int extensionIndex;

            if (askForSaving() == 1)
                return;


            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                sourceRichTextBox.ReadOnly = false;
                saveToolStripButton.Enabled = true;

                fullFileName = openFileDialog1.FileName;
                extensionIndex = fullFileName.LastIndexOf(".");
                fileName = fullFileName.Remove(extensionIndex);
                
                extension = fullFileName.Substring(extensionIndex + 1);
                asmOpened = (extension == "s" || extension == "asm");


                sourceRichTextBox.LoadFile(fullFileName, RichTextBoxStreamType.PlainText);

                alreadySaved = true;
                sourceTextChanged = false;
                allowFunctionsInToolStripForSaved();

                setGuiText(fullFileName);
                OpenLstFile(fileName + ".lst");
                projectSourcesTabControl.SelectedIndex = 0;
            }
        }
        private void compileToolStripButton_Click(object sender, EventArgs e)
        {

            string stderr = "";

            bool startAssembler = false;

            int NumOfReplacements = 0;
            int index = 0;

            bool found;
            string target;
            string lstLine;
            string asmLine;
            string farbranch_targets = String.Empty;

            System.IO.StreamReader lstSr;
            System.IO.StreamWriter sw;

#if  debug
            cf.ConsoleWriteLine("Compile started");
#endif
            if (fileName.Length == 0)
            {
                MessageBox.Show("Open a C file before compiling",
                                "Compiler Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation);
            }
            else
            {
                if (askForSaving() == 1)
                    return;

                if (asmOpened)
                    startAssembler = true;
                else
                {
                    processInit("Compiling " + fileName.Substring(fileName.LastIndexOf("\\") + 1) + ".c");

                    compilerProcess.StartInfo.Arguments = "-Wf-g,; -S " + fileName + ".c";
                    compilerProcess.Start();
                    //WriteLineToConsole("---- Compiler started (lcc-xr16.exe) ----");
                    stderr = compilerProcess.StandardError.ReadToEnd();
                    //WriteLineToConsole(stderr);
                    compilerProcess.Close();
                    if (stderr.Length != 0)
                    {
                        MessageBox.Show("Compiler messages:\r\n" + stderr,
                                    "Compiler Error",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error);
                        projectSourcesTabControl.SelectedIndex = 0;
                    }
                    else
                    {
                        startAssembler = true;
                        processDone();
                    }
                }

                if (startAssembler)
                {

                    removeLocInfoFromAssembly();



                    processInit("Assembling " + fileName.Substring(fileName.LastIndexOf("\\") + 1) + ".s");

                    assemblerProcess.StartInfo.Arguments = CRT3 + LIBXR16 + fileName + ".s" +
                                                           " -lst=" + fileName + ".lst" +
                                                           " -hex=" + fileName + ".hex";
#if  debug
                    cf.ConsoleWriteLine("Assembler started xr16.exe\r\nArguments:");
                    cf.ConsoleWriteLine(assemblerProcess.StartInfo.Arguments);
#endif

                    assemblerProcess.Start();

                    stderr = assemblerProcess.StandardError.ReadToEnd();
                    
                    assemblerProcess.Close();


                    if (stderr.Contains("branch displacement overflow at"))
                    {
                        lstSr = new System.IO.StreamReader(fileName + ".lst");
                        sw = new System.IO.StreamWriter(fileName + "_FB.s");
#if  debug
                        cf.ConsoleWriteLine("farbranch replace");
#endif
                        index = stderr.IndexOf("overflow", index);
                        index += 12;
                        target = stderr.Substring(index, 4);
                        farbranch_targets += target + "  ";
                        NumOfReplacements++;
                        found = false;

                        while (!((lstSr.ReadLine()).Contains("# file " + fileName + ".s"))) ;
                        while ((lstLine = lstSr.ReadLine()) != null)
                        {
                            if (found)
                            {
                                index = stderr.IndexOf("overflow", index);
                                if (index > 0)
                                {
                                    index += 12;
                                    target = stderr.Substring(index, 4);
                                    farbranch_targets += target + "  ";
                                    if ((++NumOfReplacements % 4) == 0)
                                        farbranch_targets += "\r\n";
                                }
                                found = false;

                            }

                            if (lstLine.StartsWith(target))
                            {
#if  debug
                                cf.ConsoleWriteLine(lstLine);
#endif
                                found = true;
                                lstLine = lstLine.Insert(lstLine.IndexOf("b", 28), "f");
#if  debug
                                cf.ConsoleWriteLine(lstLine.Substring(29));
#endif
                            }
                            asmLine = lstLine.Substring(29);

                            if (asmLine.StartsWith("#"))
                                break;
                            if (asmLine.Length > 0)
                                sw.WriteLine(asmLine);
                            if (asmLine.Contains("ret"))
                                sw.WriteLine("");
                        }

                        sw.Close();


                        assemblerProcess.StartInfo.Arguments = CRT3 + LIBXR16 + fileName + "_FB.s" +
                                                           " -lst=" + fileName + "_FB.lst" +
                                                           " -hex=" + fileName + ".hex";
#if  debug
                        cf.ConsoleWriteLine("restart assembler on" + fileName + "_FB.s");
#endif
                        assemblerProcess.Start();

                        stderr = assemblerProcess.StandardError.ReadToEnd();


                        assemblerProcess.Close();


                        MessageBox.Show("FAR_Branch inserted at:\r\n\r\n" + farbranch_targets +
                                        "\r\n\r\n(See end of the old lst file)",
                                    "Assembler Farbranch Replace",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Information);

                        OpenLstFile(fileName + "_FB.lst");
                    }
                    else
                    {
                        OpenLstFile(fileName + ".lst");
                    }
                    if (stderr.Length != 0)
                    {
                        MessageBox.Show("Assembler messages:\r\n" + stderr,
                                    "Assembler Error",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error);

                    }
                    else
                    {
                        processDone();
                        allowFunctionsInToolStripForCompiled();
                    }
                    //WriteLineToConsole("Done");

                }
            }
        }
        private void downloadToolStripButton_Click(object sender, EventArgs e)
        {
            string line;
            int i = 0;
            UInt16 data;
            UInt16 address;
            Byte hexData;
            Byte numOfData;

            int userProgramLength;

            StreamReader sr = null;

            try
            {
                //WriteLineToConsole("Download clicked");
#if  debug
                cf.ConsoleWriteLine("Download clicked");
#endif
                sr = new StreamReader(fileName + ".hex");
                FileStream fr = new FileStream(fileName + ".hex", FileMode.Open, FileAccess.Read);

#if  debug
                cf.ConsoleWriteLine("Length of hexfile:" + fr.Length.ToString());
#endif
                userProgramLength = ((int)fr.Length / 59) - 32;
                if ((int)fr.Length % 59 > 0)
                    ++userProgramLength;

#if  debug
                cf.ConsoleWriteLine("userProgramLength:" + userProgramLength.ToString());
#endif

                fr.Close();
                processInit("Downloading", userProgramLength);

                toolStrip1.Enabled = false;

                // 0008,000A-as cmen van a _main-re egy hivatkozs a fprogramban 
                line = sr.ReadLine();
                Byte.TryParse(line.Substring(9 + 3 * 8, 3), NumberStyles.HexNumber, null, out hexData);
                data = (UInt16)hexData;
                data <<= 8;
                Byte.TryParse(line.Substring(9 + 3 * 9, 3), NumberStyles.HexNumber, null, out hexData);
                data += (UInt16)hexData;

                address = 0x0008;
                debmonWriteWord(address, data);

                Byte.TryParse(line.Substring(9 + 3 * 10, 3), NumberStyles.HexNumber, null, out hexData);
                data = (UInt16)hexData;
                data <<= 8;
                Byte.TryParse(line.Substring(9 + 3 * 11, 3), NumberStyles.HexNumber, null, out hexData);
                data += (UInt16)hexData;

                address = 0x000A;
                debmonWriteWord(address, data);
                
                // Kihagyjuk az kovetkezo 8 sort mert vltozatlan
                for (i = 0; i < 8; i++)
                    sr.ReadLine();
         
                // 90 - 9E az IT vektorok cmei 
                line = sr.ReadLine();
                for (i = 0; i < 16; i += 2)
                {
                    Byte.TryParse(line.Substring(9 + 3 * i, 3), NumberStyles.HexNumber, null, out hexData);
                    data = (UInt16)hexData;
                    data <<= 8;
                    Byte.TryParse(line.Substring(9 + 3 * (i + 1), 3), NumberStyles.HexNumber, null, out hexData);
                    data += (UInt16)hexData;
                    address = (UInt16)(0x90 + i);
                    debmonWriteWord(address, data);
                }

                // Kihagyjuk az kovetkezo 22 sort
                for (i = 0; i < 22; i++)
                    sr.ReadLine();

                // 0x200-tl jn a felhasznli program
                address = 0x200;

                while ((line = sr.ReadLine()) != null)
                {

                    Byte.TryParse(line.Substring(2, 2), NumberStyles.HexNumber, null, out numOfData);
                    if ((numOfData % 2) == 1)
                    {
                        for (i = 0; i < numOfData; i++)
                        {
                            Byte.TryParse(line.Substring(9 + 3 * i, 3), NumberStyles.HexNumber, null, out hexData);

                            debmonWriteByte((UInt16)(address + i), hexData);
                        }
                    }
                    else
                    {
                        for (i = 0; i < numOfData; i += 2)
                        {
                            Byte.TryParse(line.Substring(9 + 3 * i, 3), NumberStyles.HexNumber, null, out hexData);
                            data = (UInt16)hexData;
                            data <<= 8;
                            Byte.TryParse(line.Substring(9 + 3 * (i + 1), 3), NumberStyles.HexNumber, null, out hexData);
                            data += (UInt16)hexData;

                            debmonWriteWord((UInt16)(address + i), data);
                        }

                    }
                    address += 16;
                    processToolStripProgressBar.Increment((address - 0x200) / 16);

                }

                sr.Close();
#if  debug
                cf.ConsoleWriteLine("Download complete -> reseting processor");
#endif
                debmonReset();
                processDone();
                projectSourcesTabControl.SelectedIndex = 1;
            }
            catch (FileNotFoundException)
            {
                MessageBox.Show("Not able to open HEX file",
                                "Download Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Debug serial port is not open\r\nChoose a port!  download",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                sr.Close();
            }
            finally
            {
                toolStrip1.Enabled = true;
            }



        }
        private void breakToolStripButton_Click(object sender, EventArgs e)
        {
            try
            {
                if (autoStepRunning)
                {
                    autoStepRunning = false;

                }
                else if (programRunning)
                {
                    debmonBreak();

                }

            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Debug serial port is not open\r\nChoose a port!   break",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }

        }
        private void resetToolStripButton_Click(object sender, EventArgs e)
        {
            //WriteLineToConsole("Reset clicked");

            try
            {
                processInit("Processor reset");
                debmonReset();
                processDone();
            }
            catch (InvalidOperationException)
            {

                MessageBox.Show("Debug serial port is not open\r\nChoose a port!  reset",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }


        }
        private void stepToolStripButton_Click(object sender, EventArgs e)
        {
            //WriteLineToConsole("Step clicked");
            try
            {
                processInit("Step");
                /*if(breakPointSet)
                    removeBreakpoint();
                */
                debmonStep();
                processDone();
            }
            catch (InvalidOperationException)
            {

                MessageBox.Show("Debug serial port is not open\r\nChoose a port!  step",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }
        private void autoStepToolStripButton_Click(object sender, EventArgs e)
        {
            //WriteLineToConsole("Auto Step clicked");
            autoStepTimer.Enabled = true;
            autoStepRunning = true;
            stepToolStripButton_Click(sender, e);
        }
        private void autoStepTimer_Tick(object sender, EventArgs e)
        {

            if (autoStepRunning)
            {
                if (!programRunning)
                {
#if  debug
                    cf.ConsoleWriteLine("autoStepTimer_Tick - make step");
#endif
                    stepToolStripButton_Click(new object(), EventArgs.Empty);
                }
            }
            else
            {
#if  debug
                cf.ConsoleWriteLine("autoStepTimer_Tick - stop timer");
#endif
                autoStepTimer.Enabled = false;
                setProgramHaltedGUI();
            }

        }
        private void runToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //WriteLineToConsole("Run clicked");
            processInit("Running program");
            try
            {
                if (breakPointSet)
                    setBreakpoint(breakPointAddress);
                else
                    disableBreakpoint();

                debmonContinue();

            }
            catch (InvalidOperationException)
            {
                memoryTextBox.Enabled = true;
                addrTextBox.Enabled = true;
                ioDevicesTabControl.Enabled = true;
                registersPanel.Enabled = true;

                terminalTextBox.Enabled = false;
                programRunning = false;

                MessageBox.Show("Debug serial port is not open\r\nChoose a port!   run",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }
        private void exitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(PROGRAM_NAME+" v1.00\r\n\r\nDesigned by Peter Czako",
                                    "About",
                                    MessageBoxButtons.OK);

        }
        private void iOAddressesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                ioAddrForm = new IOAddressesForm(homeDirectory);
                ioAddrForm.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.SettingsForm_FormClosing);

                this.Enabled = false;
                ioAddrForm.Show();
            }
            catch
            {
#if  debug
                cf.ConsoleWriteLine("ioAddrForm exception");
#endif
                this.Enabled = true;
                setIOGroupBoxValues();

            }
        }
        private void userIONamesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                userIONamesForm = new UserIONamesForm(homeDirectory);

                userIONamesForm.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.SettingsForm_FormClosing);

                this.Enabled = false;
                userIONamesForm.Show();
            }
            catch
            {
#if  debug
                cf.ConsoleWriteLine("userIONamesForm exception");
#endif
                this.Enabled = true;
                setIOGroupBoxValues();

            }
        }
        private void SettingsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
#if  debug
            cf.ConsoleWriteLine("SettingsForm_FormClosing");
#endif
            setIOGroupBoxValues();

            this.Enabled = true;

        }


        //----------------------SERIAL PORT + TERMINAL-----------------------------

        private void SetText(string str)
        {
            if (terminalTextBox.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetText);
                this.Invoke(d, new object[] { str });
            }
            else
            {
                terminalTextBox.Text += str;
                if (terminalTextBox.Text.Length > 4)
                {
                    terminalTextBox.Select(terminalTextBox.Text.Length - 2, 1);
                    terminalTextBox.ScrollToCaret();
                    terminalTextBox.DeselectAll();
                }

            }
        }
        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            Byte InputData;
            bool escReceived = false;

            //cf.ConsoleWriteLine("serialPort1_DataReceived");

            if (programRunning)
            {
                string serialData = serialPort1.ReadExisting();
                if (serialData.Contains(((char)27).ToString())) // Contains ESC
                {
                    escReceived = true;
                }
                SetText(serialData.Replace(((char)27).ToString(), ""));  //do not display ESC

                if (escReceived)
                {
                    programRunning = false;
                    //Thread.Sleep(1);    // Let some time to Backgroudworker1 to start running
                    escReceivedEvent.Set();

                }


            }
            else
            {
                if (readWordSet)
                {
                    InputData = (Byte)serialPort1.ReadByte();
                    receivedWord = (UInt16)InputData;
                    receivedWord <<= 8;

                    InputData = (Byte)serialPort1.ReadByte();
                    receivedWord += (UInt16)InputData;


                }
                else if (readByteSet)
                {
                    receivedByte = (Byte)serialPort1.ReadByte();

                }

                /* empty buffer */

                string rec = serialPort1.ReadExisting();

                receivedValidSerialDataEvent.Set();

            }
        }
        private void portsToolStripComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (serialPort1.IsOpen) serialPort1.Close();
                serialPort1.PortName = portsToolStripComboBox.SelectedItem.ToString();


            // try to open the selected port:
                serialPort1.Open();
            }
            // give a message, if the port is not available:
            catch
            {
                MessageBox.Show("Serial port " + serialPort1.PortName +
                   " cannot be closed or opened!", "COM Port Error",
                   MessageBoxButtons.OK, MessageBoxIcon.Warning);
                portsToolStripComboBox.SelectedText = "";

            }
        }
        private void terminalTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            char[] buffer = new char[1];
            buffer[0] = (char)e.KeyChar;
            try
            {
                if (buffer[0] == 27)
                {
                    breakToolStripButton_Click(sender, new EventArgs());
                }
                else
                {
                    serialPort1.Write(buffer, 0, 1);
                }
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Debug serial port is not open\r\nChoose a port! terminal",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }


        }
        private void portsToolStripComboBox_Click(object sender, EventArgs e)
        {
            bool found = false;
            portsToolStripComboBox.Items.Clear();
            // Nice methods to browse all available ports:
            string[] ports = SerialPort.GetPortNames();
            // Add all port names to the combo box:
            foreach (string port in ports)
            {
                portsToolStripComboBox.Items.Add(port);
                if (portsToolStripComboBox.Text == port)
                    found = true;

            }
            if (!found)
                portsToolStripComboBox.Text = "";


        }

        //----------------------BACKGROUND WORKER-----------------------------

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
#if  debug
            cf.ConsoleWriteLine("backgroundWorker1_DoWork started - waiting for ESC");
#endif
            escReceivedEvent.WaitOne();
#if  debug
            cf.ConsoleWriteLine("    backgroundWorker1_DoWork ESC catched");
#endif

            readRegisterValues();

#if  debug
            cf.ConsoleWriteLine("backgroundWorker1_DoWork - end ");
#endif
        }
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
#if  debug
            cf.ConsoleWriteLine("backgroundWorker1_RunWorkerCompleted - start");
#endif
            setRegisterValues();
            //lstRichTextBox.HideSelection = false;
            setProgramHaltedGUI();
#if  debug
            cf.ConsoleWriteLine("backgroundWorker1_RunWorkerCompleted - end");
#endif
        }

        //----------------------DEBMON COMMANDS-----------------------------

        private void debmonWriteWord(UInt16 address, UInt16 data)
        {
            Byte[] buffer = new Byte[5];

            buffer[0] = (Byte)'0';
            buffer[1] = (Byte)(address >> 8);
            buffer[2] = (Byte)address;
            buffer[3] = (Byte)(data >> 8);
            buffer[4] = (Byte)data;
            try
            {
                serialPort1.Write(buffer, 0, 5);
            }
            catch
            {
                throw;
            }
        }
        private void debmonWriteByte(UInt16 address, Byte data)
        {
            Byte[] buffer = new Byte[4];

            buffer[0] = (Byte)'1';
            buffer[1] = (Byte)(address >> 8);
            buffer[2] = (Byte)address;
            buffer[3] = data;
            try
            {
                serialPort1.Write(buffer, 0, 4);
            }
            catch
            {
                throw;
            }

        }
        private UInt16 debmonReadWord(UInt16 address)
        {
            Byte[] buffer = new Byte[3];

            readWordSet = true;

            buffer[0] = (Byte)'2';
            buffer[1] = (Byte)(address >> 8);
            buffer[2] = (Byte)address;
            try
            {
                serialPort1.Write(buffer, 0, 3);
                if(receivedValidSerialDataEvent.WaitOne(2000, false))
                    readWordSet = false;
                else
                {
                    
                    throw new Exception("TimeOut");
                }
            }
            catch
            {
                throw;
            }
            return receivedWord;


        }
        private Byte debmonReadByte(UInt16 address)
        {
            Byte[] buffer = new Byte[3];
            readByteSet = true;

            buffer[0] = (Byte)'3';
            buffer[1] = (Byte)(address >> 8);
            buffer[2] = (Byte)(address);
            try
            {
                serialPort1.Write(buffer, 0, 3);
                if(receivedValidSerialDataEvent.WaitOne(2000,false))
                    readByteSet = false;
                else
                {                    
                    throw new Exception("TimeOut");
                }
            }
            catch
            {
                throw;
            }
            return receivedByte;


        }
        private void debmonContinue()
        {
            char[] buffer = new char[1];
            buffer[0] = '4';
            try
            {
                if (!backgroundWorker1.IsBusy)
                {
#if  debug
                    cf.ConsoleWriteLine("debmonContinue()");
#endif
                    setProgramStartedGUI();
                    backgroundWorker1.RunWorkerAsync();
                    Thread.Sleep(1);
                    serialPort1.Write(buffer, 0, 1);
                }

            }
            catch
            {
                throw;
            }
        }
        private void debmonBreak()
        {
            Byte[] buffer = new Byte[1];
            buffer[0] = 27; // ESC ascii kdja
            try
            {
#if  debug
                cf.ConsoleWriteLine("debmonBreak()");
#endif
                serialPort1.Write(buffer, 0, 1);
                //debmonStartedEvent.Reset();
                //debmonStartedEvent.WaitOne();
                //setProgramHaltedGUI();

            }
            catch
            {
                throw;
            }

        }
        private void debmonReset()
        {

            try
            {
                if (!backgroundWorker1.IsBusy && !programRunning)
                {
#if  debug
                    cf.ConsoleWriteLine("debmonReset()");
#endif
                    // R14 = 0 -> restart program
                    debmonWriteWord((UInt16)SAVE_REG_ADDR+(14-1)*2, 0);
                    debmonContinue();


                }
            }
            catch
            {
                throw;
            }
        }
        private void debmonStep()
        {
            try
            {
                if (!backgroundWorker1.IsBusy && !programRunning)
                {
                    // BP_match <= R14
                    setBreakpoint(regs[14-1].RegValue);
                    debmonContinue();
                    //debmonStartedEvent.Reset();
                    //debmonStartedEvent.WaitOne();
                    //setProgramHaltedGUI(false);
                }
            }
            catch
            {
                throw;
            }


        }

        //-----------------------MISC----------------------------

        private void setProgramHaltedGUI()
        {
            string it_return_addr = String.Empty;

            bool found = false;
            int last_ch_index;

#if  debug
            cf.ConsoleWriteLine("setProgramHaltedGUI()");
#endif
            memoryTextBox.Enabled = true;
            addrTextBox.Enabled = true;
            ioDevicesTabControl.Enabled = true;
            registersPanel.Enabled = true;
            sourceRichTextBox.ReadOnly = false;


            projectSourcesTabControl.SelectedIndex = 1;

            updateIOTabValues(activeTabIndex);

            if (!autoStepRunning)
            {
                breakToolStripButton.Enabled = false;


                runToolStripButton.Enabled = true;
                stepToolStripButton.Enabled = true;
                autoStepToolStripButton.Enabled = true;
                resetToolStripButton.Enabled = true;
                downloadToolStripButton.Enabled = true;
                newToolStripButton.Enabled = true;
                openToolStripButton.Enabled = true;
                saveToolStripButton.Enabled = true;
                compileToolStripButton.Enabled = true;


                breakToolStripMenuItem.Enabled = false;
                runToolStripMenuItem.Enabled = true;
                stepToolStripMenuItem.Enabled = true;
                autoStepToolStripMenuItem.Enabled = true;
                resetToolStripMenuItem.Enabled = true;
                downloadToolStripMenuItem.Enabled = true;
                newToolStripMenuItem.Enabled = true;
                openToolStripMenuItem.Enabled = true;
                saveToolStripMenuItem.Enabled = true;
                saveAsToolStripMenuItem.Enabled = true;
                compileToolStripMenuItem.Enabled = true;

            }

            last_ch_index = lstRichTextBox.Text.IndexOf("\n", findIndex);
            lstRichTextBox.Select(findIndex, last_ch_index - findIndex);
            if (steppedOnBreakPoint)
                lstRichTextBox.SelectionBackColor = Color.DarkRed;
            else
                lstRichTextBox.SelectionBackColor = lstRichTextBox.BackColor;

            it_return_addr = regs[13].RegValue.ToString("X04");

            findIndex = 0;
            while (!found)
            {
                findIndex = lstRichTextBox.Find(it_return_addr, findIndex + 1, RichTextBoxFinds.None);
                actualLine = lstRichTextBox.GetLineFromCharIndex(findIndex);
                if (findIndex == lstRichTextBox.GetFirstCharIndexFromLine(actualLine))
                    found = true;
            }


            //lstRichTextBox.ScrollToCaret();
            //cf.ConsoleWriteLine("Find: " + it_return_addr + " found at: " + find_index.ToString());
            last_ch_index = lstRichTextBox.Text.IndexOf("\n", findIndex);
            lstRichTextBox.Select(findIndex, last_ch_index - findIndex);
            if (breakPointLine != actualLine)
            {
                lstRichTextBox.SelectionBackColor = Color.Yellow;
                steppedOnBreakPoint = false;
            }
            else
            {
                lstRichTextBox.SelectionBackColor = Color.DarkOrange;
                steppedOnBreakPoint = true;
            }

            //lstRichTextBox.Enabled = false;
            lstRichTextBox.DeselectAll();
            lstRichTextBox.Focus();

            //lstRichTextBox.Enabled = true;
            //lstRichTextBox.ScrollToCaret


            terminalTextBox.ReadOnly = true;


            //readRegisterValues();

        }
        private void setProgramStartedGUI()
        {
#if  debug
            cf.ConsoleWriteLine("setProgramStartedGUI()");
#endif
            programRunning = true;

            memoryTextBox.Enabled = false;
            addrTextBox.Enabled = false;
            ioDevicesTabControl.Enabled = false;
            registersPanel.Enabled = false;
            
            sourceRichTextBox.ReadOnly = true;
            
            newToolStripButton.Enabled = false;

            

            breakToolStripButton.Enabled = true;
            runToolStripButton.Enabled = false;
            stepToolStripButton.Enabled = false;
            autoStepToolStripButton.Enabled = false;
            resetToolStripButton.Enabled = false;
            downloadToolStripButton.Enabled = false;
            newToolStripButton.Enabled = false;
            openToolStripButton.Enabled = false;
            saveToolStripButton.Enabled = false;
            compileToolStripButton.Enabled = false;


            breakToolStripMenuItem.Enabled = true;
            runToolStripMenuItem.Enabled = false;
            stepToolStripMenuItem.Enabled = false;
            autoStepToolStripMenuItem.Enabled = false;
            resetToolStripMenuItem.Enabled = false;
            downloadToolStripMenuItem.Enabled = false;
            newToolStripMenuItem.Enabled = false;
            openToolStripMenuItem.Enabled = false;
            saveToolStripMenuItem.Enabled = false;
            saveAsToolStripMenuItem.Enabled = false;
            compileToolStripMenuItem.Enabled = false;



            terminalTextBox.ReadOnly = false;



        }
        private void readRegisterValues()
        {

            for (int j = 0; j < 15; j++)
            {
                regValues[j] = debmonReadWord((UInt16)(SAVE_REG_ADDR + j * 2));

            }
        }
        private void setRegisterValues()
        {
            for (int j = 0; j < 15; j++)
            {

                if (regs[j].RegValue != regValues[j])
                {
                    regs[j].RegTextBox.Font = new Font(
                                                    regs[j].RegTextBox.Font.FontFamily,
                                                    regs[j].RegTextBox.Font.SizeInPoints,
                                                    FontStyle.Bold
                                                    );
                }
                else
                {
                    regs[j].RegTextBox.Font = new Font(
                                                       regs[j].RegTextBox.Font.FontFamily,
                                                       regs[j].RegTextBox.Font.SizeInPoints
                                                        );
                }
                regs[j].RegValue = regValues[j];

            }

        }
        private void readMemoryValues()
        {
            int i, j;
            int firstChar;
            UInt16 getWord;
            Byte getByte;

            try
            {
                processInit("Reading Memory", 256);
#if  debug
                cf.ConsoleWriteLine("Reading memory from " + baseAddr.ToString("X04"));
#endif
                for (i = 0; i < memoryTextBox.Lines.Length; i++)
                {

                    firstChar = memoryTextBox.GetFirstCharIndexFromLine(i);

                    if (wordRadioButton.Checked)
                    {
                        for (j = 0; j < 4; j++)
                        {
                            getWord = debmonReadWord((UInt16)(baseAddr + i * 8 + j * 2));
                            memoryTextBox.Text = memoryTextBox.Text.Remove(firstChar + 6 + j * 5, 4);
                            memoryTextBox.Text = memoryTextBox.Text.Insert(
                                            firstChar + 6 + j * 5,
                                            getWord.ToString("X04")
                                            );
                            processToolStripProgressBar.Increment(2);
                        }
                    }
                    else
                    {
                        for (j = 0; j < 8; j++)
                        {
                            getByte = debmonReadByte((UInt16)(baseAddr + i * 8 + j));
                            memoryTextBox.Text = memoryTextBox.Text.Remove(firstChar + 6 + j * 3, 2);
                            memoryTextBox.Text = memoryTextBox.Text.Insert(
                                            firstChar + 6 + j * 3,
                                            getByte.ToString("X02")
                                            );
                            processToolStripProgressBar.Increment(1);
                        }

                    }

                    memoryTextBox.Text = memoryTextBox.Text.Remove(firstChar, 4);
                    memoryTextBox.Text = memoryTextBox.Text.Insert(
                                            memoryTextBox.GetFirstCharIndexFromLine(i),
                                            ((UInt16)(baseAddr + i * 8)).ToString("X04"));

                }
                processDone();
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Debug serial port is not open\r\nChoose a port!",
                    "Serial Port Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            catch
            {
                MessageBox.Show("Serial Port timed out\r\nCheck Port settings",
                                "Serial Port Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
        }
        private void processInit(string text, int maxValue)
        {
            processToolStripStatusLabel.Text = text;
            processToolStripProgressBar.Maximum = maxValue;
            processToolStripProgressBar.Value = processToolStripProgressBar.Minimum;
            statusStrip1.Refresh();
        }
        private void processInit(string text)
        {
            processToolStripStatusLabel.Text = text;
            processToolStripProgressBar.Maximum = 100;
            processToolStripProgressBar.Value = processToolStripProgressBar.Minimum;
            statusStrip1.Refresh();
        }
        private void processDone()
        {
            processToolStripProgressBar.Value = processToolStripProgressBar.Maximum;
            processToolStripStatusLabel.Text += " ---> Done";
        }
        private void setBreakpoint(UInt16 bp)
        {
            debmonWriteWord(BP_LOGIC_BASE_ADDR, bp); // SET BP
            debmonWriteByte((UInt16)(BP_LOGIC_BASE_ADDR+2), 1); // ENABLE

#if  debug
            cf.ConsoleWriteLine("           setBreakpoint to " + bp.ToString("X04"));
#endif
        }
        private void disableBreakpoint()
        {
            debmonWriteByte((UInt16)(BP_LOGIC_BASE_ADDR + 2), 0); // DISABLE BP
#if  debug
            cf.ConsoleWriteLine("           disableBreakpoint");
#endif
        }

        private void sourceRichTextBox_TextChanged(object sender, EventArgs e)
        {
            if (!sourceTextChanged)
            {
                this.Text += "*";
                sourceTextChanged = true;
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();

           // If the file name is not an empty string open it for saving.
            if (saveFileDialog1.FileName != "")
            {
                sourceRichTextBox.SaveFile(saveFileDialog1.FileName, RichTextBoxStreamType.PlainText);
                alreadySaved = true;
                sourceTextChanged = false;
                              


                fullFileName = saveFileDialog1.FileName;
                int extensionIndex = fullFileName.LastIndexOf(".");
                fileName = fullFileName.Remove(extensionIndex);

                string extension = fullFileName.Substring(extensionIndex + 1);
                asmOpened = (extension == "s" || extension == "asm");

                setGuiText(fullFileName);
                lstRichTextBox.Clear();
                lstRichTextBox.Text = "\r\n\r\n                    LST File";
                allowFunctionsInToolStripForSaved();
            }

        }

        private void setGuiText(string FileName)
        {
            this.Text = PROGRAM_NAME + " - " + FileName.Substring(FileName.LastIndexOf("\\") + 1);
        }

        private int askForSaving()
        {
            if (sourceTextChanged)
            {
                DialogResult dr = MessageBox.Show("Do you want to save changes made to " + this.Text.Substring(25, this.Text.Length - 1 - 25) + " ?",
                                                  PROGRAM_NAME,
                                                  MessageBoxButtons.YesNoCancel,
                                                  MessageBoxIcon.Warning
                                                  );
                if (dr == DialogResult.Yes)
                    saveToolStripButton_Click(new Object(),EventArgs.Empty);
                else if (dr == DialogResult.Cancel)
                    return 1;
            }
            return 0;
        }


        private void memoryTextBox_VScroll(object sender, EventArgs e)
        {
            editorTextBox.Visible = false;
        }

        private void pasteToolStripButton_Click(object sender, EventArgs e)
        {
            sourceRichTextBox.Paste(DataFormats.GetFormat(DataFormats.Text));
        }

        private void copyToolStripButton_Click(object sender, EventArgs e)
        {
            sourceRichTextBox.Copy();
        }

        private void cutToolStripButton_Click(object sender, EventArgs e)
        {
            sourceRichTextBox.Cut();
        }

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sourceRichTextBox.Undo();
        }

        private void redoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sourceRichTextBox.Redo();
        }


        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sourceRichTextBox.SelectAll();
        }

        private void sourceRichTextBox_Leave(object sender, EventArgs e)
        {
            pasteToolStripButton.Enabled = false;
            copyToolStripButton.Enabled = false;
            cutToolStripButton.Enabled = false;
            editToolStripMenuItem.Enabled = false;
        }

        private void sourceRichTextBox_Enter(object sender, EventArgs e)
        {
            if (!programRunning)
            {
                pasteToolStripButton.Enabled = true;
                copyToolStripButton.Enabled = true;
                cutToolStripButton.Enabled = true;
                editToolStripMenuItem.Enabled = true;
            }
        }

        private void projectSourcesTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (projectSourcesTabControl.SelectedIndex == 0)
                sourceRichTextBox.Focus();
        }

        private void guiForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (askForSaving() == 1)
                e.Cancel = true ;
        }

        private void sourceRichTextBox_MouseClick(object sender, MouseEventArgs e)
        {
            int line;
            int column;
            
            line = sourceRichTextBox.GetLineFromCharIndex(sourceRichTextBox.SelectionStart);
            column = sourceRichTextBox.SelectionStart - sourceRichTextBox.GetFirstCharIndexFromLine(line);
            LineNumToolStripLabel.Text = (line+1).ToString();
            ColumnNumToolStripLabel.Text = (column+1).ToString();
        }

        private void sourceRichTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            int line;
            int column;

            line = sourceRichTextBox.GetLineFromCharIndex(sourceRichTextBox.SelectionStart);
            column = sourceRichTextBox.SelectionStart - sourceRichTextBox.GetFirstCharIndexFromLine(line);
            LineNumToolStripLabel.Text = (line + 1).ToString();
            ColumnNumToolStripLabel.Text = (column + 1).ToString();
        }

        private void allowFunctionsInToolStripForNotSaved()
        {
            
            compileToolStripButton.Enabled = false;
            downloadToolStripButton.Enabled = false;

            breakToolStripButton.Enabled = false;
            runToolStripButton.Enabled = false;
            stepToolStripButton.Enabled = false;
            autoStepToolStripButton.Enabled = false;
            resetToolStripButton.Enabled = false;

            compileToolStripMenuItem.Enabled = false;
            downloadToolStripMenuItem.Enabled = false;

            breakToolStripMenuItem.Enabled = false;
            runToolStripMenuItem.Enabled = false;
            stepToolStripMenuItem.Enabled = false;
            autoStepToolStripMenuItem.Enabled = false;
            resetToolStripMenuItem.Enabled = false;
        }

        private void allowFunctionsInToolStripForSaved()
        {
            
            compileToolStripButton.Enabled = true;
            downloadToolStripButton.Enabled = false;
            
            breakToolStripButton.Enabled = false;
            runToolStripButton.Enabled = false;
            stepToolStripButton.Enabled = false;
            autoStepToolStripButton.Enabled = false;
            resetToolStripButton.Enabled = false;

            compileToolStripMenuItem.Enabled = true;
            downloadToolStripMenuItem.Enabled = false;
            
            breakToolStripMenuItem.Enabled = false;
            runToolStripMenuItem.Enabled = false;
            stepToolStripMenuItem.Enabled = false;
            autoStepToolStripMenuItem.Enabled = false;
            resetToolStripMenuItem.Enabled = false;
        }

        private void allowFunctionsInToolStripForCompiled()
        {
            
            compileToolStripButton.Enabled = true;
            downloadToolStripButton.Enabled = true;

            breakToolStripButton.Enabled = false;
            runToolStripButton.Enabled = false;
            stepToolStripButton.Enabled = false;
            autoStepToolStripButton.Enabled = false;
            resetToolStripButton.Enabled = false;

            compileToolStripMenuItem.Enabled = true;
            downloadToolStripMenuItem.Enabled = true;

            breakToolStripMenuItem.Enabled = false;
            runToolStripMenuItem.Enabled = false;
            stepToolStripMenuItem.Enabled = false;
            autoStepToolStripMenuItem.Enabled = false;
            resetToolStripMenuItem.Enabled = false;
        }
               

    } // public partial class guiForm : Form

}//namespace XSOC_Gui
