using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Globalization;

namespace XSOC_Gui
{
    public partial class IOAddressesForm : Form
    {
        string homeDirectory;
        //guiForm gf;
        TextBox[] tboxes = new TextBox[9];
        Label[] userIONames = new Label[4];
        
        public IOAddressesForm(string homeDir)
        {
            int i,j;
            InitializeComponent();
            this.homeDirectory = homeDir;
            
            tboxes[0] = SwitchSetAddrTextBox;
            tboxes[1] = LEDSetAddrTextBox;
            tboxes[2] = SegmentSetAddrTextBox;
            tboxes[3] = SRAMSetAddrTextBox;
            tboxes[4] = ITCSetAddrTextBox;
            tboxes[5] = UserIO1SetAddrTextBox;
            tboxes[6] = UserIO2SetAddrTextBox;
            tboxes[7] = UserIO3SetAddrTextBox;
            tboxes[8] = UserIO4SetAddrTextBox;

            userIONames[0] = userIO1NameLabel;
            userIONames[1] = userIO2NameLabel;
            userIONames[2] = userIO3NameLabel;
            userIONames[3] = userIO4NameLabel;
            
            try
            {
                FileStream datFile = new FileStream(homeDirectory + "\\xsoc.dat", FileMode.Open, FileAccess.ReadWrite);
                BinaryReader br = new BinaryReader(datFile);

                // Addresses
                for (i = 0; i < 9; i++)
                {
                    tboxes[i].Text = br.ReadUInt16().ToString("X04");
                }
                for (i = 0; i < 4; i++)
                {
                    userIONames[i].Text = br.ReadString() + ":";
                    // skip other  user io values
                    for (j = 0; j < 4; j++)
                    {
                        br.ReadString(); br.ReadBoolean(); br.ReadBoolean();
                    }
                }
                
                br.Close();
                datFile.Close();
            }

            catch
            {
                this.Close() ;
            }

        }

        
        private void OKButton_Click(object sender, EventArgs e)
        {
            try
            {

                FileStream datFile = new FileStream(this.homeDirectory + "\\xsoc.dat", FileMode.Open, FileAccess.Write);
                BinaryWriter w = new BinaryWriter(datFile);
                int i;
                bool[] error = new bool[9];

                for (i = 0; i < 9; i++)
                    error[i] = setDatFileAddress(tboxes[i], w);


                w.Close();
                datFile.Close();

                string errorString = String.Empty;
                string tboxName;
                bool errorFound = false;
                for (i = 0; i < 9; i++)
                {
                    if (error[i])
                    {
                        tboxName = tboxes[i].Name.Substring(0, tboxes[i].Name.IndexOf("SetAddrTextBox"));
                        errorString += "Error at " + tboxName + "\r\n";
                        errorFound = true;
                    }

                }


                //gf.setIOGroupBoxValues();

                if (errorFound)
                    MessageBox.Show(errorString,
                                    "Wrong Address",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error
                                    );
                else
                {
                    this.Close();
                }
            }
            catch
            {
                this.Close();
            }
        }

        private bool setDatFileAddress(TextBox tb, BinaryWriter bw)
        {
            UInt16 data;
            if (UInt16.TryParse(tb.Text, NumberStyles.HexNumber, null, out data))
            {
                bw.Write(data);
                return false;
            }
            else
            {
                bw.Seek(2, SeekOrigin.Current);
                tb.Text = "";
                return true;
            }

        }

        private void CancelButton1_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        
        
        private void IOAddressesForm_Leave(object sender, EventArgs e)
        {
            this.Focus();
        }

        
        

        

    }



}