using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Globalization;


namespace XSOC_Gui
{
    public partial class UserIONamesForm : Form
    {
        private string[,] Names = new string[4, 5];
        private bool[,] readOnlyArray = new bool[4, 4];
        private bool[,] sizeWordArray = new bool[4, 4];
        private UInt16[] addresses = new ushort[4];
        private int selectedIndex;
        string homeDirectory;

        public UserIONamesForm(string homeDir)
        {
            InitializeComponent();
            this.homeDirectory = homeDir;

            try
            {
                getDataFromXsocDatFile();

                for (int i = 0; i < 4; i++)
                {
                    userIOSelectComboBox.Items.Add(Names[i, 0]);

                }
                setActiveIOValues(0);
                userIOSelectComboBox.SelectedIndex = 0;
            }
            catch
            {
                this.Close();
            }

        }

        private void getDataFromXsocDatFile()
        {
            int i;
            try
            {
                FileStream datFile = new FileStream(homeDirectory + "\\xsoc.dat", FileMode.Open, FileAccess.Read);
                BinaryReader br = new BinaryReader(datFile);

                br.ReadBytes(10);   // Skip standard address values
                for (i = 0; i < 4; i++)
                    addresses[i] = br.ReadUInt16();

                for (i = 0; i < 4; i++)
                {
                    Names[i, 0] = br.ReadString();
                    for (int j = 0; j < 4; j++)
                    {
                        Names[i, j + 1] = br.ReadString();
                        readOnlyArray[i, j] = br.ReadBoolean();
                        sizeWordArray[i, j] = br.ReadBoolean();
                    }
                }

                br.Close();
                datFile.Close();
            }
            catch
            {
                throw;
            }
        }

        private void setDataInXsocDatFile()
        {
            int i;
            try
            {
                FileStream datFile = new FileStream(homeDirectory + "\\xsoc.dat", FileMode.Open, FileAccess.Write);
                BinaryWriter bw = new BinaryWriter(datFile);

                bw.Seek(10, SeekOrigin.Begin);   // Skip address values
                for (i = 0; i < 4; i++)
                    bw.Write(addresses[i]);
                
                for (i = 0; i < 4; i++)
                {
                    bw.Write(Names[i, 0]);
                    for (int j = 0; j < 4; j++)
                    {
                        bw.Write(Names[i, j + 1]);
                        bw.Write(readOnlyArray[i, j]);
                        bw.Write(sizeWordArray[i, j]);
                    }
                }

                bw.Close();
                datFile.Close();
            }
            catch
            {
                this.Close();
            }
        }
        private void storeActiveIOValues()
        {
            UInt16 setValue;

            if (UInt16.TryParse(AddrTextBox.Text, NumberStyles.HexNumber, null, out setValue))
                addresses[selectedIndex] = setValue;
                       
            Names[selectedIndex, 1] = reg1NameTextBox.Text;
            Names[selectedIndex, 2] = reg2NameTextBox.Text;
            Names[selectedIndex, 3] = reg3NameTextBox.Text;
            Names[selectedIndex, 4] = reg4NameTextBox.Text;

            readOnlyArray[selectedIndex, 0] = reg1ReadOnlyCheckBox.Checked;
            readOnlyArray[selectedIndex, 1] = reg2ReadOnlyCheckBox.Checked;
            readOnlyArray[selectedIndex, 2] = reg3ReadOnlyCheckBox.Checked;
            readOnlyArray[selectedIndex, 3] = reg4ReadOnlyCheckBox.Checked;

            sizeWordArray[selectedIndex, 0] = reg1Word.Checked;
            sizeWordArray[selectedIndex, 1] = reg2Word.Checked;
            sizeWordArray[selectedIndex, 2] = reg3Word.Checked;
            sizeWordArray[selectedIndex, 3] = reg4Word.Checked;
        }

        private void setAddressOffsetLabels()
        {
            int index = reg1Word.Checked ? 2 : 1;

            if (reg2Word.Checked && (index % 2 == 1))
                index++;
            reg2AddrLabel.Text = "+" + index.ToString();
            index += reg2Word.Checked ? 2 : 1;
            
            if (reg3Word.Checked && (index % 2 == 1))
                index++;
            reg3AddrLabel.Text = "+" + index.ToString();
            index += reg3Word.Checked ? 2 : 1;
            
            if (reg4Word.Checked && (index % 2 == 1))
                index++;
            reg4AddrLabel.Text = "+" + index.ToString();
            


        }
        private void setActiveIOValues(int UserIONum)
        {
            AddrTextBox.Text = addresses[UserIONum].ToString("X04");

            reg1NameTextBox.Text = Names[UserIONum, 1];
            reg2NameTextBox.Text = Names[UserIONum, 2];
            reg3NameTextBox.Text = Names[UserIONum, 3];
            reg4NameTextBox.Text = Names[UserIONum, 4];

            reg1ReadOnlyCheckBox.Checked = readOnlyArray[UserIONum, 0];
            reg2ReadOnlyCheckBox.Checked = readOnlyArray[UserIONum, 1];
            reg3ReadOnlyCheckBox.Checked = readOnlyArray[UserIONum, 2];
            reg4ReadOnlyCheckBox.Checked = readOnlyArray[UserIONum, 3];

            reg1Byte.Checked = !(reg1Word.Checked = sizeWordArray[UserIONum, 0]) ;
            reg2Byte.Checked = !(reg2Word.Checked = sizeWordArray[UserIONum, 1]);
            reg3Byte.Checked = !(reg3Word.Checked = sizeWordArray[UserIONum, 2]);
            reg4Byte.Checked = !(reg4Word.Checked = sizeWordArray[UserIONum, 3]);
            setAddressOffsetLabels();
        }

        private void userIOSelectComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            storeActiveIOValues();
            setActiveIOValues(userIOSelectComboBox.SelectedIndex);
            selectedIndex = userIOSelectComboBox.SelectedIndex;
        }

        private void OKButton_Click(object sender, EventArgs e)
        {
            storeActiveIOValues();
            setDataInXsocDatFile();
            this.Close();
        }

        private void CancelButton1_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void userIOSelectComboBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (userIOSelectComboBox.SelectedIndex == -1)
                {
                    Names[selectedIndex, 0] = userIOSelectComboBox.Text;
                    userIOSelectComboBox.Items.Insert(selectedIndex, userIOSelectComboBox.Text);
                    userIOSelectComboBox.Items.RemoveAt(selectedIndex + 1);
                }
                           
            }
        }

        private void regWord_CheckedChanged(object sender, EventArgs e)
        {
            setAddressOffsetLabels();
        }

        private void AddrTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            UInt16 setValue;
            if (e.KeyChar == (char)Keys.Enter)
            {

                if (UInt16.TryParse(AddrTextBox.Text, NumberStyles.HexNumber, null, out setValue))
                {
                    addresses[selectedIndex] = setValue;
                    AddrTextBox.Text = setValue.ToString("X04");
                }
                else
                {
                    AddrTextBox.Text = addresses[selectedIndex].ToString("X04");
                }
                
            }
        }



    }
}