;
; CRT3.s -- xr16 C runtime library startup
; Created by: Peter Czako
;
; This file must be assembled first, so that reset and interrupt receive
; the correct addresses:
; 0000 reset vector
; 0010 interrupt vector;
; 
; Info:
;
; - SP initialized to 0x0FFE
; - 8 Interrupts
; - Breakpoint set at the start of _main function -> automatic IRQ at start
; - Needs Interrupt controller and Breakpoint Logic to work properly
;

global _main
global _isr0
global _isr1
global _isr2
global _isr3
global _isr4
global _isr5
global _isr6
global _isr7

align 16
reset:   
   lea sp,0x0ffe
   lea r6,0x6000   ; ITC_ADDRESS
   lea r7,_main
   lea r1,1
   br init
align 16
interrupt:
   ; save registers
   sw r1,saveregs
   lea r1,saveregs
   sw r2,2(r1)
   sw r3,4(r1)
   sw r4,6(r1)
   sw r5,8(r1)
   sw r6,10(r1)
   sw r7,12(r1)
   sw r8,14(r1)
   sw r9,16(r1)
   sw r10,18(r1)
   sw r11,20(r1)
   sw r12,22(r1)
   sw r13,24(r1)
   sw r14,26(r1)   ; r14 is the (reserved) interrupt return address
   sw r15,28(r1)
   
   lea r8,it_vect
   lw r6,0x6000   ; ITC vector address
   or r6,r8
   lw r7,0(r6)
   slli r7, 4   ; r7 contains -> (C | ITvectAddr[15:4]) -> r7 << 4
   jal r15,0(r7)   ; 

   ; reload registers
   lea r1,saveregs
   lw r2,2(r1)
   lw r3,4(r1)
   lw r4,6(r1)
   lw r5,8(r1)
   lw r6,10(r1)
   lw r7,12(r1)
   lw r8,14(r1)
   lw r9,16(r1)
   lw r10,18(r1)
   lw r11,20(r1)
   lw r12,22(r1)
   lw r13,24(r1)
   lw r14,26(r1) ; r14 is the (reserved) interrupt return address
   lw r15,28(r1) 
   lw r1,saveregs

   ; return from interrupt
   reti

init:   
   sw r7,4(r6)   ; BP_MATCH_REG <= _main
   sb r1,2(r6)   ; IT_ENABLE
   sb r1,6(r6)   ; BP_ENABLE
   jal r0,0(r7)   ; j _main
saveregs:
   bss 30


align 16
it_vect :
   call _isr0
   call _isr1
   call _isr2
   call _isr3
   call _isr4
   call _isr5
   call _isr6
   call _isr7

align 16
_isr0:
_isr1:
_isr2:
_isr3:
_isr4:
_isr5:
_isr6:
_isr7:
   ret