/* ---------------------------------------------------
  Name: libxr16.c 
  Copyright: 
  Author: Peter Czako
  Date: 24.11.06 19:46
  Description:  A minimal runtime library for xr16 and XSOC
  				Contains UART receive, transmit and print functions
				and a simple debugger monitor which runs from isr0. 
  				
  ----------------------------------------------------------				
*/

#include "xsoc.h"

void debmon(void);

char uart_getc (){
	while (!( UART_RX_STATUS & 0x04));	/* data_present */
	return UART_RX;
} /* uart_getc */

void uart_putc(char ch_tx){
	while (UART_TX_STATUS & 2 ); /*buffer full*/
	UART_TX = ch_tx;
} /* uart_putc */

void uart_print (const char* string){
	int i=0;
	while (string[i] != 0){
		uart_putc(string[i++]);
	}
} /* uart_print vge */

void isr0(){
	BP_STATUS = 0;
	UART_RX_STATUS  = 0;
	debmon();
	UART_RX_STATUS  = 8;
}
/* Function can be optimized in assembly */
void debmon(){
	char ch;
	int addr;
	/* debmon started */
	uart_putc(27); /* SEND ESC */
	while (1){
		ch = uart_getc(); /* WAIT FOR COMMANDS*/
		if ('0' <= ch && ch <= '3'){
			addr = ((int)uart_getc() << 8) | uart_getc();
			if(ch == '0') /* write word */
				*(int*)addr = ((int)uart_getc() << 8) | uart_getc();
			else if(ch == '1') /* write byte */
				*(char*)addr = uart_getc();
			else if(ch == '2'){ /* read word */
				uart_putc((char)(*(int*)addr >> 8));
				uart_putc((char)(*(int*)(addr)));	
			}else /* read byte */
				uart_putc(*(char*)addr);
		}
		else if (ch == '4'){ /* continue */
			return;
		}
	}
}
