/*
-----------------------------------------------------------------------

  Name: xsoc.h
  Copyright: 
  Author: Peter Czak
  Date: 23.04.07 20:58
  Description: Header file for the XSOC system
  			   Defines macros for accessing the peripherals and special 
  			   characters for the 7-segment display
-----------------------------------------------------------------------
*/

#ifndef _XSOC_MD
#define _XSOC_MD

/* INTERRUPT CONTROLLER */
#define ITC_VECT *(unsigned int*)0x6000
#define ITC_EN *(char*)0x6002
#define ITC_STATUS *(char*)0x6003

/* BREAKPOINT LOGIC */
#define BP_MATCH *(int*)0x6004
#define BP_STATUS *(char*)0x6006

/* XSOC 8 BIT I/O */
#define SWITCH *(char*)0x6020
#define LED *(char*)0x6030

/* 7 SEGMENT DISPLAY */
#define SEGM_AN0 *(char*)0x6040
#define SEGM_AN1 *(char*)0x6041 
#define SEGM_AN2 *(char*)0x6042
#define SEGM_AN3 *(char*)0x6043
#define SEGM_HEX *(unsigned int*)0x6044 
#define SEGM_SEL *(char*)0x6046 

/* XSOC UART */
#define UART_TX *(char*)0x6050
#define UART_TX_STATUS *(char*)0x6051
#define UART_RX *(char*)0x6052
#define UART_RX_STATUS *(char*)0x6053

extern char uart_getc(void);
extern void uart_putc(char ch_tx);
extern void uart_print(const char* string);

/* SRAM CONTROLLER */
#define MEM_PAGE *(char*)0x6060

/* XSOC TIMER16 */
#define TIMER_MATCH *(unsigned int*)0x6070
#define TIMER_CNT   *(unsigned int*)0x6072
#define TIMER_CTRL  *(char*)0x6074

/* 7 SEGMENT LETTERS */
/* Low active control pins
*       _7_   
*     2|   |6  
*      |_1_| 
*      |   |
*     3|_4_|5 .0
*
*/

#define LET_A (char)0x11
#define LET_B (char)0xC1
#define LET_C (char)0x63
#define LET_D (char)0x85
#define LET_E (char)0x61
#define LET_F (char)0x71

/*      _   */
/*G =  |    */
/*     |_|  */

#define LET_G (char)0x43

#define LET_H (char)0xD1
#define LET_I (char)0xF3
#define LET_J (char)0x87

/*      _    */
/*K =  |_    */
/*     | |   */

#define LET_K (char)0x51

#define LET_L (char)0xE3

/*      _    */
/*M =  | |   */
/*     | |   */


#define LET_M (char)0x13

#define LET_N (char)0xD5
#define LET_O (char)0xC5
#define LET_P (char)0x31
#define LET_Q (char)0x19
#define LET_R (char)0xF5
#define LET_S (char)0x49
#define LET_T (char)0xE1
#define LET_U (char)0xC7

/*         */
/*V = | |  */
/*    |_|  */

#define LET_V (char)0x83

/*          */
/*W1 = |    */
/*     |_|.  */
/*          */
/*W2 =   |  */
/*     |_|  */


#define LET_W1 (char)0xC2
#define LET_W2 (char)0x87
#define LET_X (char)0x91
#define LET_Y (char)0x89
#define LET_Z (char)0x25
#define LET_SP (char)0xFF

#define NUM_0 (char)0x03
#define NUM_1 (char)0x9F
#define NUM_2 (char)0x25
#define NUM_3 (char)0x0D
#define NUM_4 (char)0x99
#define NUM_5 (char)0x49
#define NUM_6 (char)0x41
#define NUM_7 (char)0x1F
#define NUM_8 (char)0x01
#define NUM_9 (char)0x09

#endif /* _XSOC_MD */
