`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:         BME-VIK
// Engineer:        Peter Czako
// 
// Create Date:     06/27/2006  
// Design Name: 
// Module Name:     anode_sel 
// Project Name:    XSOC
// Target Devices:  SPARTAN-3, SPARTAN-3E
// Tool versions:   XILINX ISE 8.2
// Description:     Submodule of the 7 segment display controller.
//                  Generates the low active anode selection outputs.
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//=========================================================
// MODULE DECLARATION
//=========================================================
module anode_sel (
    input               clk,        // 12 MHz input clock
    input               rst,        // Global reset
    input               clk_ce,     // 1 kHz display refresh clock enable
    output  reg [3:0]   anode,      // 7 segment display anode control signals
    output      [1:0]   an_2bit     // Select signal for the segment multiplexer
    );


// ASSIGNS
    assign an_2bit [1]  = ~anode[3] | ~anode[2];
    assign an_2bit [0]  = ~anode[3] | ~anode[1];

//=========================================================
// CIRCULAR SHIFT REGISTER
//=========================================================
    always @(posedge clk) begin
        if (rst)
            anode <= 4'b0111;
        else if(clk_ce)
            anode <= {anode[0],anode[3:1]};
    end

endmodule    
