`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:         BME-VIK
// Engineer:        Peter Czako
// 
// Create Date:     07/11/2006  
// Design Name: 
// Module Name:     baud_clk 
// Project Name:    XSOC
// Target Devices:  SPARTAN-3, SPARTAN-3E
// Tool versions:   XILINX ISE 8.2
// Description:     Baud clock generation module for the XSOC UART. Generates 
//                  clock enable pulses at a 16x higher frequency than the
//                  actual baud rate.
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//=========================================================
// MODULE DECLARATION
//=========================================================
module baud_clk(
    input   clk,      // 12 MHz input clk
    input   rst,      // Global reset
    output  baud      // Baud rate clock pulses
    );

//=========================================================     
//
// 57600 BAUD CLOCK GENERATION FROM 12 MHz
// 
// 12 MHz / (16 * 57600)    = 13,0208 -> 13 
// THIS WAY THE BAUD IS 57692 Hz WHICH IS IN THE 1% DIFFERENCE DOMAIN
// 
// OTHER BAUD CLOCK SETTINGS
// 9600 - 78,125     => 78  (9615  (0,16%))
// 19200 - 39,062    => 39  (19230 (0,16%))
// 38400 -  19,53    => 20  (37500 (3,45%)) !!!
// 57600 -  13,02    => 13  (57692 (0,16%))
// 115200 - 6,51     => 7   (107142(3,45%)) !!!
//
//=========================================================

// PARAMETERS
    parameter BAUD_DIV_9600  = 78;   
    parameter BAUD_DIV_19200 = 39;
    parameter BAUD_DIV_57600 = 13;

// REGS
    reg [6:0] cnt;      // Clock generation counter


//=========================================================
// BAUD CLOCK PULSE GENERATION 
//=========================================================
    always @ (posedge clk)  begin
        if (rst | cnt == (BAUD_DIV_57600-1))
            cnt <= 0;
        else
            cnt <= cnt + 1;
    end

assign baud = cnt == (BAUD_DIV_57600-1);

endmodule
