`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:         BME-VIK
// Engineer:        Peter Czako
// 
// Create Date:     19:09:41 08/23/2007
// Design Name: 
// Module Name:     bus_gate 
// Project Name:    XSOC
// Target Devices:  SPARTAN-3, SPARTAN-3E
// Tool versions:   XILINX ISE 8.2
// Description:     Separate module for the gating of the XSOC bus signals
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//=========================================================
// MODULE DECLARATION
//=========================================================
module bus_gate(
    input           rnw_m0_i,       // Master 0 read/~write
    input   [1:0]   be_m0_i,        // Master 0 byte enable
    input   [15:0]  addr_m0_i,      // Master 0 address bus
    
    input           rnw_m1_i,       // Master 1 read/~write
    input   [1:0]   be_m1_i,        // Master 1 byte enable
    input   [15:0]  addr_m1_i,      // Master 1 address bus
    
    input   [15:0]  dout0_i,        // Both master and slave data out signals
    input   [15:0]  dout1_i,
    input   [15:0]  dout2_i,
    input   [15:0]  dout3_i,
    input   [15:0]  dout4_i,
    input   [15:0]  dout5_i,
    input   [15:0]  dout6_i,
    input   [15:0]  dout7_i,
    input   [15:0]  dout8_i,
    input   [15:0]  dout9_i,
    input   [15:0]  dout10_i,
    input   [15:0]  dout11_i,
    
    output          rnw_o,          // Common read/~write
    output  [1:0]   be_o,           // Common byte enable
    output  [15:0]  addr_o,         // Common address bus
    output  [15:0]  din_o           // Common data in bus
    );
    
//=========================================================
// ASSIGNS
//=========================================================

// Inactive state = 1
    assign be_o     = be_m0_i & be_m1_i;            
    assign rnw_o    = rnw_m0_i & rnw_m1_i;  
    
// Inactive state = 0
    assign addr_o   = addr_m0_i | addr_m1_i;
    assign din_o    = dout0_i | dout1_i | dout2_i | dout3_i | 
                      dout4_i | dout5_i | dout6_i | dout7_i |
                      dout8_i | dout9_i | dout10_i | dout11_i;


endmodule
