`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:         BME-VIK
// Engineer:        Peter Czako
// 
// Create Date:     20:26:32 06/20/2006  
// Design Name: 
// Module Name:     clk_div 
// Project Name:    XSOC
// Target Devices:  SPARTAN-3, SPARTAN-3E
// Tool versions:   XILINX ISE 8.2
// Description:     Clock divider module which generates the appropriate
//                  12 MHz XSOC clock from the external oscillator frequency
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
 
//=========================================================
// MODULE DECLARATION
//=========================================================

module clk_div(
    input   clk_i,           // Clock from board oscillator
    output  clk_o            // 12 MHz XSOC system clock
    );   

//=========================================================
// PARAMETERS           
//=========================================================
         
// SPARTAN-3 Starter Board : 50 MHz -> 12 MHz ( 50 * 6/25 )
    parameter DIV = 25;
    parameter MUL = 6; 

//=========================================================
// SUBMODULES
//=========================================================
    
// DCM: Digital Clock Manager Circuit
//      Virtex-II/II-Pro and Spartan-3
// Xilinx HDL Language Template, version 8.2.2i

   DCM #(
      .CLKDV_DIVIDE(2.0),                   // Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5
                                            //   7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
      .CLKFX_DIVIDE(DIV),                   // Can be any integer from 1 to 32
      .CLKFX_MULTIPLY(MUL),                 // Can be any integer from 2 to 32
      .CLKIN_DIVIDE_BY_2("FALSE"),          // TRUE/FALSE to enable CLKIN divide by two feature
      .CLKIN_PERIOD(20.0),                  // Specify period of input clock
      .CLKOUT_PHASE_SHIFT("NONE"),          // Specify phase shift of NONE, FIXED or VARIABLE
      .CLK_FEEDBACK("1X"),                  // Specify clock feedback of NONE, 1X or 2X
      .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"), // SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or
                                            //   an integer from 0 to 15
      .DFS_FREQUENCY_MODE("LOW"),           // HIGH or LOW frequency mode for frequency synthesis
      .DLL_FREQUENCY_MODE("LOW"),           // HIGH or LOW frequency mode for DLL
      .DUTY_CYCLE_CORRECTION("TRUE"),       // Duty cycle correction, TRUE or FALSE
      .FACTORY_JF(16'hC080),                // FACTORY JF values
      .PHASE_SHIFT(0),                      // Amount of fixed phase shift from -255 to 255
      .STARTUP_WAIT("TRUE")                 // Delay configuration DONE until DCM LOCK, TRUE/FALSE
   ) DCM_inst (
      .CLK0(CLK0),     // 0 degree DCM CLK output
      .CLK180(),       // 180 degree DCM CLK output
      .CLK270(),       // 270 degree DCM CLK output
      .CLK2X(),        // 2X DCM CLK output
      .CLK2X180(),     // 2X, 180 degree DCM CLK out
      .CLK90(),        // 90 degree DCM CLK output
      .CLKDV(),        // Divided DCM CLK out (CLKDV_DIVIDE)
      .CLKFX(clk_o),   // DCM CLK synthesis out (M/D)
      .CLKFX180(),     // 180 degree CLK synthesis out
      .LOCKED(),       // DCM LOCK status output
      .PSDONE(),       // Dynamic phase adjust done output
      .STATUS(),       // 8-bit DCM status bits output
      .CLKFB(CLK0),    // DCM clock feedback
      .CLKIN(clk_i),   // Clock input (from IBUFG, BUFG or DCM)
      .PSCLK(1'b0),    // Dynamic phase adjust clock input
      .PSEN(1'b0),     // Dynamic phase adjust enable input
      .PSINCDEC(1'b0), // Dynamic phase adjust increment/decrement
      .RST(1'b0)       // DCM asynchronous reset input
   );

// End of DCM_inst instantiation


endmodule
