`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:         BME-VIK
// Engineer:        Peter Czako
// 
// Create Date:     06/27/2006  
// Design Name: 
// Module Name:     xsoc_timer16 
// Project Name:    XSOC
// Target Devices:  SPARTAN-3, SPARTAN-3E
// Tool versions:   XILINX ISE 8.2
// Description:     Submodule for the 7 segment display controller.
//                  A ROM which stores the appropriate segment control values
//                  for the hexadecimal numbers.
//                  Low active control bits: 
//                      _7_
//                     |   |
//                    2|_1_|6
//                     |   |
//                    3|_4_|5 .0
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//=========================================================
// MODULE DECLARATION
//=========================================================

module seg_table (
    input       [3:0] datain,
    output reg  [7:0] segctrl
    );

    always @(datain) begin
        case (datain)
            4'b0000: segctrl <= 8'b00000011;    // 0:  3
            4'b0001: segctrl <= 8'b10011111;    // 1:159
            4'b0010: segctrl <= 8'b00100101;    // 2: 37    
            4'b0011: segctrl <= 8'b00001101;    // 3: 13
            4'b0100: segctrl <= 8'b10011001;    // 4:153
            4'b0101: segctrl <= 8'b01001001;    // 5: 73
            4'b0110: segctrl <= 8'b01000001;    // 6: 65
            4'b0111: segctrl <= 8'b00011111;    // 7: 31
            4'b1000: segctrl <= 8'b00000001;    // 8:  1
            4'b1001: segctrl <= 8'b00001001;    // 9:  9
            4'b1010: segctrl <= 8'b00010001;    // A: 17
            4'b1011: segctrl <= 8'b11000001;    // b:193
            4'b1100: segctrl <= 8'b01100011;    // C: 99    
            4'b1101: segctrl <= 8'b10000101;    // d:133
            4'b1110: segctrl <= 8'b01100001;    // E: 97
            4'b1111: segctrl <= 8'b01110001;    // F:113
            default: segctrl <= 8'b11111111;    // Blank display
        endcase
    end
endmodule
