`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:         BME-VIK
// Engineer:        Peter Czako
// 
// Create Date:     14:31:41 10/23/2006 
// Design Name: 
// Module Name:     sign_mux 
// Project Name:    XSOC
// Target Devices:  SPARTAN-3, SPARTAN-3E
// Tool versions:   XILINX ISE 8.2
// Description:     Additional module for the xr16 processor's datapath unit.
//                  Serves the sign generation for barrel shift operations.
//                  Needs to be placed after the multiplier but before the 
//                  result multiplexer in the datapath module.
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//=========================================================
// MODULE DECLARATION
//=========================================================

module sign_mux(
    input           sign,       // Sign bit, 1-negative 0-positive
    input           error,      // Shift error indicator
    input           arit,       // Arithmetic shift operation
    input   [3:0]   ammount,    // Shift ammount
    output  [15:0]  res         // Sign extension result
    );

//=========================================================
// SIGN GENERATION
//=========================================================
     
    reg [15:0]  out;           // Stores the sign extension

    // SIGN EXTENSION ONLY NEEDED IN CASE OF ARITHMETIC SHIFT AND IF
    // THE OPERAND WAS NEGATIVE. IF ERROR OCCURED DURING SHIFTING
    // -1 (0xFFFF) IS RETURNED
    assign res = (~sign | ~arit) ? 16'h0000 : (error ? 16'hFFFF : out);
    
    always @ (ammount) begin
        case (ammount)
            4'd0 : out = 16'h0000;
            4'd1 : out = 16'h8000;
            4'd2 : out = 16'hC000;
            4'd3 : out = 16'hE000;
            4'd4 : out = 16'hF000;
            4'd5 : out = 16'hF800;
            4'd6 : out = 16'hFC00;
            4'd7 : out = 16'hFE00;
            4'd8 : out = 16'hFF00;
            4'd9 : out = 16'hFF80;
            4'd10: out = 16'hFFC0;
            4'd11: out = 16'hFFE0;
            4'd12: out = 16'hFFF0;
            4'd13: out = 16'hFFF8;
            4'd14: out = 16'hFFFC;
            4'd15: out = 16'hFFFE;
            default : out = 16'h0000;
        endcase
    end

endmodule

