`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:         BME-VIK
// Engineer:        Peter Czako
// 
// Create Date:     06/29/2006  
// Design Name: 
// Module Name:     xsoc_out8 
// Project Name:    XSOC
// Target Devices:  SPARTAN-3, SPARTAN-3E
// Tool versions:   XILINX ISE 8.2
// Description:     Simple 8 bit output controller for the XSOC system.
//                  Used for connect any 8 bit peripheral (eg. LEDs) to
//                  the XSOC bus.
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

//=========================================================
// MODULE DECLARATION
//=========================================================
 
module xsoc_out8(
// XSOC BUS INTERFACE
    input           clk,
    input           rst,
    input           rnw,
    input   [1:0]   be,
    input   [15:0]  addr,
    input   [15:0]  din,
    output  [15:0]  dout,
// USER IO PORTS
    output reg  [7:0]   out8        // Registered 8 bit output
    );
    
//=========================================================
// XSOC BUS INTERFACE LOGIC 
//=========================================================
    parameter   BASE_ADDRESS = 16'h6030;

    reg [15:0]  to_dout;
    
    wire        sel;
    

    assign sel          = addr[15:1] == BASE_ADDRESS >> 1;     
    assign dout[7:0]    = ~clk & sel & rnw & be[0] ? to_dout[7:0] : 8'b0;
    assign dout[15:8]   = ~clk & sel & rnw & be[1] ? to_dout[15:8] : 8'b0;
    
//=========================================================
// USER LOGIC
//=========================================================

//========================================================= 
// WRITE REGISTERS
//========================================================= 
    always @(negedge clk) begin
        if (rst)
            out8 <= 0;
        else if ( sel & ~rnw) begin
            if (be[1] & ~be[0])
                out8 <= din[15:8];
            else if (~be[1] & be[0])
                out8 <= din[7:0];
        end
    end
    
//========================================================= 
// READ REGISTERS
//=========================================================
    always @(negedge clk) begin
        if (rst)
            to_dout <= 0;
        else if (sel & rnw)
            to_dout <= {out8,out8};
        else
            to_dout <=0;
    end
    
endmodule 
